@@ -2,7 +2,7 @@
  * ncat_listen.c -- --listen mode.                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *
- * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
  * Project"). Nmap is also a registered trademark of the Nmap Project.
  *
  * This program is distributed under the terms of the Nmap Public Source
@@ -37,15 +37,16 @@
  * right to know exactly what a program is going to do before they run it.
  * This also allows you to audit the software for security holes.
  *
- * Source code also allows you to port Nmap to new platforms, fix bugs, and add
- * new features. You are highly encouraged to submit your changes as a Github PR
- * or by email to the dev@nmap.org mailing list for possible incorporation into
- * the main distribution. Unless you specify otherwise, it is understood that
- * you are offering us very broad rights to use your submissions as described in
- * the Nmap Public Source License Contributor Agreement. This is important
- * because we fund the project by selling licenses with various terms, and also
- * because the inability to relicense code has caused devastating problems for
- * other Free Software projects (such as KDE and NASM).
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
  *
  * The free version of Nmap is distributed in the hope that it will be
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,7 +56,7 @@
  *
  ***************************************************************************/
 
-/* $Id: ncat_listen.c 38653 2023-04-14 17:11:46Z dmiller $ */
+/* $Id: ncat_listen.c 39089 2025-03-05 17:16:12Z dmiller $ */
 
 #include "ncat.h"
 
@@ -93,6 +94,16 @@
 #define SHUT_WR SD_SEND
 #endif
 
+#ifdef WIN32
+/* Using fselect() converts STDIN_FILENO to a socket with WSA_FLAG_OVERLAPPED,
+ * so read() doesn't work. Instead, we can use recv(). */
+#define READ_STDIN(_buf, _len) (recv(_get_osfhandle(STDIN_FILENO), _buf, _len, 0))
+#define READ_STDIN_ERR() logdebug("Error reading from stdin: %08x\n", WSAGetLastError())
+#else
+#define READ_STDIN(_buf, _len) (read(STDIN_FILENO, _buf, _len))
+#define READ_STDIN_ERR() logdebug("Error reading from stdin: %s\n", strerror(errno))
+#endif
+
 /* read_fds is the clients we are accepting data from. broadcast_fds is the
    clients were are sending data to. broadcast_fds doesn't include the listening
    socket and stdin. Network clients are not added to read_fds when --send-only
@@ -116,7 +127,7 @@ static int stdin_eof = 0;
 static int crlf_state = 0;
 
 static void handle_connection(int socket_accept, int type, fd_set *listen_fds);
-static int read_stdin(void);
+static int read_stdin(struct timeval *qtv);
 static int read_socket(int recv_fd);
 static void post_handle_connection(struct fdinfo *sinfo);
 static void close_fd(struct fdinfo *fdn, int eof);
@@ -192,7 +203,7 @@ int ncat_listen()
 {
     int rc, i, j, fds_ready;
     fd_set listen_fds;
-    struct timeval tv;
+    struct timeval tv={0}, qtv={0};
     struct timeval *tvp = NULL;
     unsigned int num_sockets;
     int proto = o.proto;
@@ -285,10 +296,9 @@ int ncat_listen()
 
     init_fdlist(&broadcast_fdlist, o.conn_limit);
 
-    if (o.idletimeout > 0)
-        tvp = &tv;
-
     while (client_fdlist.nfds > 1 || get_conn_count() > 0) {
+        long usec_wait = -1;
+        tvp = NULL;
         /* We pass these temporary descriptor sets to fselect, since fselect
            modifies the sets it receives. */
         fd_set readfds = master_readfds, writefds = master_writefds;
@@ -300,20 +310,31 @@ int ncat_listen()
         if (o.debug > 1 && o.broker)
             logdebug("Broker connection count is %d\n", get_conn_count());
 
-        if (o.idletimeout > 0)
-            ms_to_timeval(tvp, o.idletimeout);
+        if (stdin_eof && o.quitafter > 0) {
+            struct timeval now;
+            gettimeofday(&now, 0);
+            usec_wait = TIMEVAL_SUBTRACT(qtv, now);
+            if (usec_wait < 0)
+                usec_wait = 0;
+        }
 
         /* The idle timer should only be running when there are active connections */
-        if (get_conn_count())
-            fds_ready = fselect(client_fdlist.fdmax + 1, &readfds, &writefds, NULL, tvp);
-        else
-            fds_ready = fselect(client_fdlist.fdmax + 1, &readfds, &writefds, NULL, NULL);
+        if (o.idletimeout > 0 && get_conn_count() && o.idletimeout * 1000 < usec_wait)
+            usec_wait = o.idletimeout * 1000;
+
+        if (usec_wait >= 0) {
+            tvp = &tv;
+            tv.tv_sec = 0;
+            tv.tv_usec = usec_wait;
+        }
+
+        fds_ready = fselect(client_fdlist.fdmax + 1, &readfds, &writefds, NULL, tvp);
 
         if (o.debug > 1)
             logdebug("select returned %d fds ready\n", fds_ready);
 
         if (fds_ready == 0)
-            bye("Idle timeout expired (%d ms).", o.idletimeout);
+            bye("Idle timeout expired (%ld ms).", usec_wait / 1000);
 
         /* If client_fdlist.state increases, the list has changed and we
          * need to go over it again. */
@@ -375,14 +396,14 @@ restart_fd_loop:
                     read_and_broadcast(cfd);
                 } else {
                     /* Read from stdin and write to all clients. */
-                    rc = read_stdin();
-                    if (rc == 0) {
-                        if (o.proto != IPPROTO_TCP || (o.proto == IPPROTO_TCP && o.sendonly)) {
+                    rc = read_stdin(&qtv);
+                    if (rc == 0 && type == SOCK_STREAM) {
+                        if (o.quitafter == 0 && (o.proto != IPPROTO_TCP || (o.proto == IPPROTO_TCP && o.sendonly))) {
                             /* There will be nothing more to send. If we're not
                                receiving anything, we can quit here. */
                             return 0;
                         }
-                        if (!o.noshutdown && type == SOCK_STREAM) shutdown_sockets(SHUT_WR);
+                        if (!o.noshutdown) shutdown_sockets(SHUT_WR);
                     }
                     if (rc < 0)
                         return 1;
@@ -439,8 +460,24 @@ static void handle_connection(int socket_accept, int type, fd_set *listen_fds)
       int nbytes = recvfrom(socket_accept, buf, sizeof(buf), MSG_PEEK,
           &s.remoteaddr.sockaddr, &s.ss_len);
       if (nbytes < 0) {
-        loguser("%s.\n", socket_strerror(socket_errno()));
-        return;
+          int err = socket_errno();
+          switch (err) {
+              // Recoverable try-again errors:
+              case EINTR:
+              case EAGAIN:
+                  loguser("%s.\n", socket_strerror(socket_errno()));
+                  return;
+              // Ignorable errors:
+              // Windows returns SOCKET_ERROR and WSAEMSGSIZE instead of truncating!
+              case EMSGSIZE:
+                  if (s.remoteaddr.sockaddr.sa_family != AF_UNSPEC)
+                      break;
+              // everything else:
+              default:
+                  die("recvfrom");
+                  return;
+                  break;
+          }
       }
       /*
        * We're using connected udp. This has the down side of only
@@ -592,7 +629,7 @@ static void close_fd(struct fdinfo *fdn, int eof) {
         logdebug("Closing connection.\n");
 #ifdef HAVE_OPENSSL
     if (o.ssl && fdn->ssl) {
-        if (eof)
+        if (eof && !o.noshutdown)
             SSL_shutdown(fdn->ssl);
         SSL_free(fdn->ssl);
     }
@@ -613,19 +650,24 @@ static void close_fd(struct fdinfo *fdn, int eof) {
 
 /* Read from stdin and broadcast to all client sockets. Return the number of
    bytes read, or -1 on error. */
-int read_stdin(void)
+int read_stdin(struct timeval *qtv)
 {
     int nbytes;
     char buf[DEFAULT_TCP_BUF_LEN];
     char *tempbuf = NULL;
 
-    nbytes = read(STDIN_FILENO, buf, sizeof(buf));
+    nbytes = READ_STDIN(buf, sizeof(buf));
     if (nbytes <= 0) {
         if (nbytes < 0 && o.verbose)
-            logdebug("Error reading from stdin: %s\n", strerror(errno));
+            READ_STDIN_ERR();
         if (nbytes == 0 && o.debug)
             logdebug("EOF on stdin\n");
 
+        if (o.quitafter > 0) {
+            struct timeval when;
+            gettimeofday(&when, 0);
+            TIMEVAL_MSEC_ADD(*qtv, when, o.quitafter);
+        }
         /* Don't close the file because that allows a socket to be fd 0. */
         checked_fd_clr(STDIN_FILENO, &master_readfds);
         /* Buf mark that we've seen EOF so it doesn't get re-added to the
@@ -712,10 +754,10 @@ static void read_and_broadcast(int recv_fd)
 
         /* Behavior differs depending on whether this is stdin or a socket. */
         if (recv_fd == STDIN_FILENO) {
-            n = read(recv_fd, buf, sizeof(buf));
+            n = READ_STDIN(buf, sizeof(buf));
             if (n <= 0) {
                 if (n < 0 && o.verbose)
-                    logdebug("Error reading from stdin: %s\n", strerror(errno));
+                    READ_STDIN_ERR();
                 if (n == 0 && o.debug)
                     logdebug("EOF on stdin\n");
 
@@ -791,7 +833,13 @@ static void shutdown_sockets(int how)
 
         fdn = get_fdinfo(&broadcast_fdlist, i);
         ncat_assert(fdn != NULL);
-        shutdown(fdn->fd, how);
+#ifdef HAVE_OPENSSL
+        if (o.ssl && fdn->ssl) {
+                SSL_shutdown(fdn->ssl);
+        }
+        else
+#endif
+            shutdown(fdn->fd, how);
     }
 }
 
