@@ -93,7 +93,7 @@ static int lzstream_docompress(lua_State *L, lz_stream *s, int from, int to, int
 
 
 static lz_stream *lzstream_new(lua_State *L, int src) {
-    lz_stream *s = (lz_stream*)lua_newuserdata(L, sizeof(lz_stream));
+    lz_stream *s = (lz_stream*)lua_newuserdatauv(L, sizeof(lz_stream), 0);
 
     luaL_getmetatable(L, ZSTREAMMETA);
     lua_setmetatable(L, -2);        /* set metatable */
@@ -163,7 +163,7 @@ static lz_stream *lzstream_check(lua_State *L, int index, int state) {
 
 static int lzstream_tostring(lua_State *L) {
     lz_stream *s = (lz_stream*)luaL_checkudata(L, 1, ZSTREAMMETA);
-    if (s == NULL) luaL_argerror(L, 1, "bad zlib stream");
+    if (s == NULL) return luaL_argerror(L, 1, "bad zlib stream");
 
     if (s->state == LZ_NONE) {
         lua_pushstring(L, "zlib stream (closed)");
