@@ -9,10 +9,9 @@
 -- @author Patrik Karlsson <patrik@cqure.net>
 --
 
-local bin = require "bin"
 local ipOps = require "ipOps"
 local match = require "match"
-local math = require "math"
+local rand = require "rand"
 local nmap = require "nmap"
 local stdnse = require "stdnse"
 local string = require "string"
@@ -50,15 +49,14 @@ Header = {
   -- @name Header.parse
   parse = function(data)
     local header = Header:new()
-    local pos
-    pos, header.type, header.length, header.trans_id = bin.unpack(">SSA16", data)
+    header.type, header.length, header.trans_id = string.unpack(">I2I2 c16", data)
     return header
   end,
 
   -- converts the header to an opaque string
   -- @return string containing the header instance
   __tostring = function(self)
-    return bin.pack(">SSA", self.type, self.length, self.trans_id)
+    return string.pack(">I2I2", self.type, self.length) .. self.trans_id
   end,
 }
 
@@ -135,14 +133,12 @@ Attribute = {
     local attr = Attribute:new()
     local pos = 1
 
-    pos, attr.type, attr.length = bin.unpack(">SS", data, pos)
+    attr.type, attr.length, pos = string.unpack(">I2I2", data, pos)
 
     local function parseAddress(data, pos)
-      local _, addr = nil, {}
-      pos, _, addr.family, addr.port, addr.ip = bin.unpack("<CCSI", data, pos)
-      if ( addr.ip ) then
-        addr.ip = ipOps.fromdword(addr.ip)
-      end
+      local addr = {}
+      addr.family, addr.port, addr.ip, pos = string.unpack(">xBI2c4", data, pos)
+      addr.ip = ipOps.str_to_ip(addr.ip)
       return addr
     end
 
@@ -155,7 +151,7 @@ Attribute = {
       end
       attr.addr = parseAddress(data, pos)
     elseif( attr.type == Attribute.SERVER ) then
-      pos, attr.server = bin.unpack("A" .. attr.length-1, data, pos)
+      attr.server = data:sub(pos, pos + attr.length - 1)
     end
 
     return attr
@@ -164,7 +160,7 @@ Attribute = {
   -- converts an attribute to string
   -- @return string containing the serialized attribute
   __tostring = function(self)
-    return bin.pack(">SSA", self.type, self.length, self.data or "")
+    return string.pack(">I2I2", self.type, self.length) .. (self.data or "")
   end,
 
 }
@@ -192,7 +188,7 @@ Response = {
     -- @name Response.Bind.parse
     parse = function(data)
       local resp = Response.Bind:new()
-      local pos = Header.size
+      local pos = Header.size + 1
 
       resp.header = Header.parse(data)
       resp.attributes = {}
@@ -252,7 +248,7 @@ Comm = {
   --         err string containing an error message, if status is false
   -- @name Comm.recv
   recv = function(self)
-    local status, hdr_data = self.socket:receive_buf(match.numbytes(Header.size), false)
+    local status, hdr_data = self.socket:receive_buf(match.numbytes(Header.size), true)
     if ( not(status) ) then
       return false, "Failed to receive response from server"
     end
@@ -262,7 +258,7 @@ Comm = {
       return false, "Failed to parse response header"
     end
 
-    local status, data = self.socket:receive_buf(match.numbytes(header.length), false)
+    local status, data = self.socket:receive_buf(match.numbytes(header.length), true)
     if ( header.type == MessageType.BINDING_RESPONSE ) then
       local resp = Response.Bind.parse(hdr_data .. data)
       return true, resp
@@ -294,21 +290,6 @@ Comm = {
   end,
 }
 
--- The Util class
-Util = {
-
-  --- creates a random string
-  -- @param len number containing the length of the generated random string
-  -- @return str containing the random string
-  -- @name Util.randomString
-  randomString = function(len)
-    local str = {}
-    for i=1, len do str[i] = string.char(math.random(255)) end
-    return table.concat(str)
-  end
-
-}
-
 -- The Helper class
 Helper = {
 
@@ -348,9 +329,9 @@ Helper = {
     local trans_id
 
     if ( self.mode == "classic" ) then
-      trans_id = Util.randomString(16)
+      trans_id = rand.random_string(16)
     else
-      trans_id = bin.pack("HA","2112A442", Util.randomString(12))
+      trans_id = "\x21\x12\xA4\x42" .. rand.random_string(12)
     end
     local req = Request.Bind:new(trans_id)
 
