@@ -3,7 +3,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *
- * The nsock parallel socket event library is (C) 1999-2023 Nmap Software LLC
+ * The nsock parallel socket event library is (C) 1999-2025 Nmap Software LLC
  * This library is free software; you may redistribute and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation; Version 2. This guarantees your right to use, modify, and
@@ -48,7 +48,7 @@
  *
  ***************************************************************************/
 
-/* $Id: engine_poll.c 38653 2023-04-14 17:11:46Z dmiller $ */
+/* $Id: engine_poll.c 39083 2025-02-26 17:44:43Z dmiller $ */
 
 #ifndef WIN32
 /* Allow the use of POLLRDHUP, if available. */
@@ -129,43 +129,21 @@ struct io_engine engine_poll = {
 /* --- INTERNAL PROTOTYPES --- */
 static void iterate_through_event_lists(struct npool *nsp);
 
-/* defined in nsock_core.c */
-void process_iod_events(struct npool *nsp, struct niod *nsi, int ev);
-void process_event(struct npool *nsp, gh_list_t *evlist, struct nevent *nse, int ev);
-void process_expired_events(struct npool *nsp);
-#if HAVE_PCAP
-#ifndef PCAP_CAN_DO_SELECT
-int pcap_read_on_nonselect(struct npool *nsp);
-#endif
-#endif
-
-/* defined in nsock_event.c */
-void update_first_events(struct nevent *nse);
-
-
-extern struct timeval nsock_tod;
-
 
 /*
  * Engine specific data structure
  */
 struct poll_engine_info {
   int capacity;
-  int max_fd;
-  /* index of the highest poll event */
+  int used;
+  int idx_insert;
+  int max_idx;
   POLLFD *events;
+  /* Number of IODs incompatible with poll */
+  int num_pcap_nonselect;
 };
 
 
-
-static inline int lower_max_fd(struct poll_engine_info *pinfo) {
-  do {
-    pinfo->max_fd--;
-  } while (pinfo->max_fd >= 0 && pinfo->events[pinfo->max_fd].fd == -1);
-
-  return pinfo->max_fd;
-}
-
 static inline int evlist_grow(struct poll_engine_info *pinfo) {
   int i;
 
@@ -194,7 +172,10 @@ int poll_init(struct npool *nsp) {
 
   pinfo = (struct poll_engine_info *)safe_malloc(sizeof(struct poll_engine_info));
   pinfo->capacity = 0;
-  pinfo->max_fd = -1;
+  pinfo->used = 0;
+  pinfo->idx_insert = 0;
+  pinfo->max_idx = -1;
+  pinfo->num_pcap_nonselect = 0;
   evlist_grow(pinfo);
 
   nsp->engine_data = (void *)pinfo;
@@ -213,58 +194,107 @@ void poll_destroy(struct npool *nsp) {
 int poll_iod_register(struct npool *nsp, struct niod *iod, struct nevent *nse, int ev) {
   struct poll_engine_info *pinfo = (struct poll_engine_info *)nsp->engine_data;
   int sd;
+  int idx;
+  POLLFD *pev;
 
   assert(!IOD_PROPGET(iod, IOD_REGISTERED));
 
   iod->watched_events = ev;
 
   sd = nsock_iod_get_sd(iod);
-  while (pinfo->capacity < sd + 1)
-    evlist_grow(pinfo);
 
-  pinfo->events[sd].fd = sd;
-  pinfo->events[sd].events = 0;
-  pinfo->events[sd].revents = 0;
+  if (sd == -1) {
+    if (iod->pcap)
+      pinfo->num_pcap_nonselect++;
+    else
+      fatal("Unable to get descriptor for IOD #%lu", iod->id);
+    iod->engine_info = -1;
+  }
+  else {
+    if (pinfo->used == pinfo->capacity)
+      evlist_grow(pinfo);
 
-  pinfo->max_fd = MAX(pinfo->max_fd, sd);
+    idx = pinfo->idx_insert;
+    while (pinfo->events[idx].fd != -1) {
+      idx = (idx + 1) % pinfo->capacity;
+      // XXX: remove this assert after thorough testing.
+      assert(idx != pinfo->idx_insert);
+    }
+    if (idx > pinfo->max_idx)
+      pinfo->max_idx = idx;
 
-  if (ev & EV_READ)
-    pinfo->events[sd].events |= POLL_R_FLAGS;
-  if (ev & EV_WRITE)
-    pinfo->events[sd].events |= POLL_W_FLAGS;
+    iod->engine_info = idx;
+    pinfo->idx_insert = (idx + 1) % pinfo->capacity;
+    pinfo->used++;
+
+    pev = &pinfo->events[idx];
+    pev->fd = sd;
+    pev->events = 0;
+    pev->revents = 0;
+
+    if (ev & EV_READ)
+      pev->events |= POLL_R_FLAGS;
+    if (ev & EV_WRITE)
+      pev->events |= POLL_W_FLAGS;
 #ifndef WIN32
-  if (ev & EV_EXCEPT)
-    pinfo->events[sd].events |= POLL_X_FLAGS;
+    if (ev & EV_EXCEPT)
+      pev->events |= POLL_X_FLAGS;
 #endif
+  }
 
   IOD_PROPSET(iod, IOD_REGISTERED);
   return 1;
 }
 
 int poll_iod_unregister(struct npool *nsp, struct niod *iod) {
+  struct poll_engine_info *pinfo = (struct poll_engine_info *)nsp->engine_data;
+  int sd, idx;
+  POLLFD *pev;
+
   iod->watched_events = EV_NONE;
 
   /* some IODs can be unregistered here if they're associated to an event that was
    * immediately completed */
   if (IOD_PROPGET(iod, IOD_REGISTERED)) {
-    struct poll_engine_info *pinfo = (struct poll_engine_info *)nsp->engine_data;
-    int sd;
 
     sd = nsock_iod_get_sd(iod);
-    pinfo->events[sd].fd = -1;
-    pinfo->events[sd].events = 0;
-    pinfo->events[sd].revents = 0;
+    if (sd == -1) {
+      assert(iod->pcap);
+      pinfo->num_pcap_nonselect--;
+    }
+    else {
+      idx = iod->engine_info;
+      assert(idx >= 0 && idx <= pinfo->max_idx);
+      pev = &pinfo->events[idx];
+      assert(pev->fd == sd);
+      iod->engine_info = -1;
 
-    if (pinfo->max_fd == sd)
-      lower_max_fd(pinfo);
+      pev->fd = -1;
+      pev->events = 0;
+      pev->revents = 0;
 
+      pinfo->used--;
+      if (idx == pinfo->max_idx) {
+        do {
+          pinfo->max_idx--;
+        } while (pinfo->max_idx >= 0 && pinfo->events[pinfo->max_idx].fd == -1);
+      }
+      if (idx < pinfo->idx_insert) {
+        pinfo->idx_insert = idx;
+      }
+      if (pinfo->max_idx < pinfo->idx_insert) {
+        pinfo->idx_insert = pinfo->max_idx + 1;
+      }
+
+    }
     IOD_PROPCLR(iod, IOD_REGISTERED);
   }
   return 1;
 }
 
 int poll_iod_modify(struct npool *nsp, struct niod *iod, struct nevent *nse, int ev_set, int ev_clr) {
-  int sd;
+  int sd, idx;
+  POLLFD *pev;
   int new_events;
   struct poll_engine_info *pinfo = (struct poll_engine_info *)nsp->engine_data;
 
@@ -281,15 +311,20 @@ int poll_iod_modify(struct npool *nsp, struct niod *iod, struct nevent *nse, int
   iod->watched_events = new_events;
 
   sd = nsock_iod_get_sd(iod);
+  if (sd >= 0) {
+    idx = iod->engine_info;
+    assert(idx >= 0 && idx <= pinfo->max_idx);
 
-  pinfo->events[sd].fd = sd;
-  pinfo->events[sd].events = 0;
+    pev = &pinfo->events[idx];
+    pev->fd = sd;
+    pev->events = 0;
 
-  /* regenerate the current set of events for this IOD */
-  if (iod->watched_events & EV_READ)
-    pinfo->events[sd].events |= POLL_R_FLAGS;
-  if (iod->watched_events & EV_WRITE)
-    pinfo->events[sd].events |= POLL_W_FLAGS;
+    /* regenerate the current set of events for this IOD */
+    if (iod->watched_events & EV_READ)
+      pev->events |= POLL_R_FLAGS;
+    if (iod->watched_events & EV_WRITE)
+      pev->events |= POLL_W_FLAGS;
+  }
 
   return 1;
 }
@@ -299,6 +334,7 @@ int poll_loop(struct npool *nsp, int msec_timeout) {
   int event_msecs; /* msecs before an event goes off */
   int combined_msecs;
   int sock_err = 0;
+  unsigned int iod_count;
   struct poll_engine_info *pinfo = (struct poll_engine_info *)nsp->engine_data;
 
   assert(msec_timeout >= -1);
@@ -306,61 +342,61 @@ int poll_loop(struct npool *nsp, int msec_timeout) {
   if (nsp->events_pending == 0)
     return 0; /* No need to wait on 0 events ... */
 
+  iod_count = gh_list_count(&nsp->active_iods) - pinfo->num_pcap_nonselect;
   do {
     struct nevent *nse;
 
     nsock_log_debug_all("wait for events");
+    results_left = 0;
 
     nse = next_expirable_event(nsp);
     if (!nse)
       event_msecs = -1; /* None of the events specified a timeout */
-    else
-      event_msecs = MAX(0, TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod));
+    else {
+      event_msecs = TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod);
+      event_msecs = MAX(0, event_msecs);
+    }
 
 #if HAVE_PCAP
-#ifndef PCAP_CAN_DO_SELECT
-    /* Force a low timeout when capturing packets on systems where
-     * the pcap descriptor is not select()able. */
-    if (gh_list_count(&nsp->pcap_read_events) > 0)
-      if (event_msecs > PCAP_POLL_INTERVAL)
-        event_msecs = PCAP_POLL_INTERVAL;
-#endif
-#endif
+    if (pinfo->num_pcap_nonselect > 0 && gh_list_count(&nsp->pcap_read_events) > 0) {
 
+      /* do non-blocking read on pcap devices that doesn't support select()
+       * If there is anything read, just leave this loop. */
+      if (pcap_read_on_nonselect(nsp)) {
+        /* okay, something was read. */
+        // poll engine's iterate_through_event_lists() also handles pcap iods.
+        // Make the system call non-blocking
+        event_msecs = 0;
+      }
+      /* Force a low timeout when capturing packets on systems where
+       * the pcap descriptor is not select()able. */
+      else if (event_msecs > PCAP_POLL_INTERVAL) {
+        event_msecs = PCAP_POLL_INTERVAL;
+      }
+    }
+#endif
     /* We cast to unsigned because we want -1 to be very high (since it means no
      * timeout) */
     combined_msecs = MIN((unsigned)event_msecs, (unsigned)msec_timeout);
 
-#if HAVE_PCAP
-#ifndef PCAP_CAN_DO_SELECT
-    /* do non-blocking read on pcap devices that doesn't support select()
-     * If there is anything read, just leave this loop. */
-    if (pcap_read_on_nonselect(nsp)) {
-      /* okay, something was read. */
-    } else
-#endif
-#endif
-    {
-      results_left = Poll(pinfo->events, pinfo->max_fd + 1, combined_msecs);
+    if (iod_count > 0) {
+      results_left = Poll(pinfo->events, pinfo->max_idx + 1, combined_msecs);
       if (results_left == -1)
         sock_err = socket_errno();
     }
+    else if (combined_msecs > 0) {
+      // No compatible IODs; sleep the remainder of the wait time.
+      usleep(combined_msecs * 1000);
+    }
+
 
     gettimeofday(&nsock_tod, NULL); /* Due to poll delay */
   } while (results_left == -1 && sock_err == EINTR); /* repeat only if signal occurred */
 
   if (results_left == -1 && sock_err != EINTR) {
-#ifdef WIN32
-    for (int i = 0; sock_err != EINVAL || i <= pinfo->max_fd; i++) {
-      if (sock_err != EINVAL || pinfo->events[i].fd != -1) {
-#endif
-        nsock_log_error("nsock_loop error %d: %s", sock_err, socket_strerror(sock_err));
-        nsp->errnum = sock_err;
-        return -1;
-#ifdef WIN32
-      }
-    }
-#endif
+    nsock_log_error("nsock_loop error %d: %s", sock_err, socket_strerror(sock_err));
+    nsp->errnum = sock_err;
+    return -1;
   }
 
   iterate_through_event_lists(nsp);
@@ -374,21 +410,25 @@ int poll_loop(struct npool *nsp, int msec_timeout) {
 static inline int get_evmask(struct npool *nsp, struct niod *nsi) {
   struct poll_engine_info *pinfo = (struct poll_engine_info *)nsp->engine_data;
   int sd, evmask = EV_NONE;
+  int idx;
   POLLFD *pev;
 
   if (nsi->state != NSIOD_STATE_DELETED
       && nsi->events_pending
       && IOD_PROPGET(nsi, IOD_REGISTERED)) {
 
+      sd = nsock_iod_get_sd(nsi);
 #if HAVE_PCAP
-      if (nsi->pcap)
-        sd = ((mspcap *)nsi->pcap)->pcap_desc;
-      else
+      // For incompatible pcap handles, we have to just try a non-blocking read.
+      if (sd == -1 && nsi->pcap)
+        return EV_READ;
 #endif
-        sd = nsi->sd;
 
-      assert(sd < pinfo->capacity);
-      pev = &pinfo->events[sd];
+      idx = nsi->engine_info;
+
+      assert(idx >= 0 && idx <= pinfo->max_idx);
+      pev = &pinfo->events[idx];
+      assert(pev->fd == sd);
 
       if (pev->revents & POLL_R_FLAGS)
         evmask |= EV_READ;
