@@ -5,128 +5,59 @@
  * http://insecure.org/osdetect/                                           *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
 #ifndef OSSCAN2_H
@@ -139,8 +70,7 @@
 #include <vector>
 #include <list>
 #include "timing.h"
-struct FingerPrint;
-struct FingerTest;
+#include "osscan.h"
 class FingerPrintResultsIPv4;
 class Target;
 
@@ -149,8 +79,6 @@ class Target;
  * CONSTANT DEFINITIONS                                                       *
  ******************************************************************************/
 
-#define NUM_FPTESTS    13
-
 /* The number of tries we normally do.  This may be increased if
    the target looks like a good candidate for fingerprint submission, or fewer
    if the user gave the --max-os-tries option */
@@ -241,17 +169,12 @@ typedef enum OFProbeType {
  * FUNCTION PROTOTYPES                                                        *
  ******************************************************************************/
 
-/* This is the primary OS detection function.  If many Targets are
-   passed in (the threshold is based on timing level), they are
-   processed as smaller groups to improve accuracy  */
-void os_scan2(std::vector<Target *> &Targets);
-
 int get_initial_ttl_guess(u8 ttl);
 
 int identify_sequence(int numSamples, u32 *ipid_diffs, int islocalhost, int allipideqz);
-int get_diffs(u32 *ipid_diffs, int numSamples, u32 *ipids, int islocalhost);
-int get_ipid_sequence_16(int numSamples, u32 *ipids, int islocalhost);
-int get_ipid_sequence_32(int numSamples, u32 *ipids, int islocalhost);
+int get_diffs(u32 *ipid_diffs, int numSamples, const u32 *ipids, int islocalhost);
+int get_ipid_sequence_16(int numSamples, const u32 *ipids, int islocalhost);
+int get_ipid_sequence_32(int numSamples, const u32 *ipids, int islocalhost);
 
 const char *ipidclass2ascii(int seqclass);
 const char *tsseqclass2ascii(int seqclass);
@@ -278,7 +201,7 @@ class OFProbe {
   OFProbe();
 
   /* The literal string for the current probe type. */
-  const char *typestr();
+  const char *typestr() const;
 
   /* Type of the probe: for what os fingerprinting test? */
   OFProbeType type;
@@ -317,9 +240,9 @@ class HostOsScanStats {
   std::list<OFProbe *>::iterator getActiveProbe(OFProbeType type, int subid);
   void moveProbeToActiveList(std::list<OFProbe *>::iterator probeI);
   void moveProbeToUnSendList(std::list<OFProbe *>::iterator probeI);
-  unsigned int numProbesToSend() {return probesToSend.size();}
-  unsigned int numProbesActive() {return probesActive.size();}
-  FingerPrint *getFP() {return FP;}
+  unsigned int numProbesToSend() const {return probesToSend.size();}
+  unsigned int numProbesActive() const {return probesActive.size();}
+  FingerPrint *getFP() const {return FP;}
 
   Target *target; /* the Target */
   struct seq_info si;
@@ -343,8 +266,7 @@ class HostOsScanStats {
    * and the last one.  Zero is
    * returned if we didn't send the tseq probes because there was no
    * open tcp port */
-  double timingRatio();
-  double cc_scale();
+  double timingRatio() const;
 
  private:
   /* Ports of the targets used in os fingerprinting. */
@@ -372,22 +294,22 @@ class HostOsScanStats {
    * finally be passed to hs->target->FPR->FPs[x]. */
   FingerPrint *FP;
   FingerTest *FPtests[NUM_FPTESTS];
-  #define FP_TSeq  FPtests[0]
-  #define FP_TOps  FPtests[1]
-  #define FP_TWin  FPtests[2]
-  #define FP_TEcn  FPtests[3]
-  #define FP_T1_7_OFF 4
-  #define FP_T1    FPtests[4]
-  #define FP_T2    FPtests[5]
-  #define FP_T3    FPtests[6]
-  #define FP_T4    FPtests[7]
-  #define FP_T5    FPtests[8]
-  #define FP_T6    FPtests[9]
-  #define FP_T7    FPtests[10]
-  #define FP_TUdp  FPtests[11]
-  #define FP_TIcmp FPtests[12]
-  struct AVal *TOps_AVs[6]; /* 6 AVs of TOps */
-  struct AVal *TWin_AVs[6]; /* 6 AVs of TWin */
+  #define FP_TSeq  FPtests[ID2INT(FingerPrintDef::SEQ)]
+  #define FP_TOps  FPtests[ID2INT(FingerPrintDef::OPS)]
+  #define FP_TWin  FPtests[ID2INT(FingerPrintDef::WIN)]
+  #define FP_TEcn  FPtests[ID2INT(FingerPrintDef::ECN)]
+  #define FP_T1_7_OFF ID2INT(FingerPrintDef::T1)
+  #define FP_T1    FPtests[ID2INT(FingerPrintDef::T1)]
+  #define FP_T2    FPtests[ID2INT(FingerPrintDef::T2)]
+  #define FP_T3    FPtests[ID2INT(FingerPrintDef::T3)]
+  #define FP_T4    FPtests[ID2INT(FingerPrintDef::T4)]
+  #define FP_T5    FPtests[ID2INT(FingerPrintDef::T5)]
+  #define FP_T6    FPtests[ID2INT(FingerPrintDef::T6)]
+  #define FP_T7    FPtests[ID2INT(FingerPrintDef::T7)]
+  #define FP_TUdp  FPtests[ID2INT(FingerPrintDef::U1)]
+  #define FP_TIcmp FPtests[ID2INT(FingerPrintDef::IE)]
+  const char *TOps_AVs[6]; /* 6 AVs of TOps */
+  const char *TWin_AVs[6]; /* 6 AVs of TWin */
 
   /* The following are variables to store temporary results
    * during the os fingerprinting process of this host. */
@@ -408,8 +330,7 @@ class ScanStats {
 
  public:
   ScanStats();
-  bool sendOK(); /* Returns true if the system says that sending is OK. */
-  double cc_scale();
+  bool sendOK() const; /* Returns true if the system says that sending is OK. */
 
   struct ultra_timing_vals timing;
   struct timeout_info to;      /* rtt/timeout info                */
@@ -443,7 +364,7 @@ class HostOsScan {
   void sendNextProbe(HostOsScanStats *hss);
 
   /* Process one response. If the response is useful, return true. */
-  bool processResp(HostOsScanStats *hss, struct ip *ip, unsigned int len, struct timeval *rcvdtime);
+  bool processResp(HostOsScanStats *hss, const struct ip *ip, unsigned int len, struct timeval *rcvdtime);
 
   /* Make up the fingerprint. */
   void makeFP(HostOsScanStats *hss);
@@ -451,27 +372,27 @@ class HostOsScan {
   /* Check whether the host is sendok. If not, fill _when_ with the
    * time when it will be sendOK and return false; else, fill it with
    * now and return true. */
-  bool hostSendOK(HostOsScanStats *hss, struct timeval *when);
+  bool hostSendOK(HostOsScanStats *hss, struct timeval *when) const;
 
   /* Check whether it is ok to send the next seq probe to the host. If
    * not, fill _when_ with the time when it will be sendOK and return
    * false; else, fill it with now and return true. */
-  bool hostSeqSendOK(HostOsScanStats *hss, struct timeval *when);
+  bool hostSeqSendOK(HostOsScanStats *hss, struct timeval *when) const;
 
 
   /* How long I am currently willing to wait for a probe response
    * before considering it timed out.  Uses the host values from
    * target if they are available, otherwise from gstats.  Results
    * returned in MICROseconds.  */
-  unsigned long timeProbeTimeout(HostOsScanStats *hss);
+  unsigned long timeProbeTimeout(HostOsScanStats *hss) const;
 
   /* If there are pending probe timeouts, fills in when with the time
    * of the earliest one and returns true.  Otherwise returns false
    * and puts now in when. */
-  bool nextTimeout(HostOsScanStats *hss, struct timeval *when);
+  bool nextTimeout(HostOsScanStats *hss, struct timeval *when) const;
 
   /* Adjust various timing variables based on pcket receipt. */
-  void adjust_times(HostOsScanStats *hss, OFProbe *probe, struct timeval *rcvdtime);
+  void adjust_times(HostOsScanStats *hss, const OFProbe *probe, const struct timeval *rcvdtime);
 
 private:
   /* Probe send functions. */
@@ -482,13 +403,13 @@ private:
   void sendTUdpProbe(HostOsScanStats *hss, int probeNo);
   void sendTIcmpProbe(HostOsScanStats *hss, int probeNo);
   /* Response process functions. */
-  bool processTSeqResp(HostOsScanStats *hss, struct ip *ip, int replyNo);
-  bool processTOpsResp(HostOsScanStats *hss, struct tcp_hdr *tcp, int replyNo);
-  bool processTWinResp(HostOsScanStats *hss, struct tcp_hdr *tcp, int replyNo);
-  bool processTEcnResp(HostOsScanStats *hss, struct ip *ip);
-  bool processT1_7Resp(HostOsScanStats *hss, struct ip *ip, int replyNo);
-  bool processTUdpResp(HostOsScanStats *hss, struct ip *ip);
-  bool processTIcmpResp(HostOsScanStats *hss, struct ip *ip, int replyNo);
+  bool processTSeqResp(HostOsScanStats *hss, const struct ip *ip, int replyNo);
+  bool processTOpsResp(HostOsScanStats *hss, const struct tcp_hdr *tcp, int replyNo);
+  bool processTWinResp(HostOsScanStats *hss, const struct tcp_hdr *tcp, int replyNo);
+  bool processTEcnResp(HostOsScanStats *hss, const struct ip *ip);
+  bool processT1_7Resp(HostOsScanStats *hss, const struct ip *ip, int replyNo);
+  bool processTUdpResp(HostOsScanStats *hss, const struct ip *ip);
+  bool processTIcmpResp(HostOsScanStats *hss, const struct ip *ip, int replyNo);
 
   /* Generic sending functions used by the above probe functions. */
   int send_tcp_probe(HostOsScanStats *hss,
@@ -507,7 +428,7 @@ private:
   void makeTOpsFP(HostOsScanStats *hss);
   void makeTWinFP(HostOsScanStats *hss);
 
-  bool get_tcpopt_string(struct tcp_hdr *tcp, int mss, char *result, int maxlen);
+  int get_tcpopt_string(const struct tcp_hdr *tcp, int mss, char *result, int maxlen) const;
 
   int rawsd;    /* Raw socket descriptor */
   eth_t *ethsd; /* Ethernet handle       */
@@ -540,8 +461,8 @@ class OsScanInfo {
    * then add to it again, or you may mess up nextI (I'm not sure) */
   std::list<HostOsScanInfo *> incompleteHosts;
 
-  unsigned int numIncompleteHosts() {return incompleteHosts.size();}
-  HostOsScanInfo *findIncompleteHost(struct sockaddr_storage *ss);
+  unsigned int numIncompleteHosts() const {return incompleteHosts.size();}
+  HostOsScanInfo *findIncompleteHost(const struct sockaddr_storage *ss);
 
   /* A circular buffer of the incompleteHosts.  nextIncompleteHost() gives
      the next one.  The first time it is called, it will give the
