@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *
- * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
  * Project"). Nmap is also a registered trademark of the Nmap Project.
  *
  * This program is distributed under the terms of the Nmap Public Source
@@ -41,15 +41,16 @@
  * right to know exactly what a program is going to do before they run it.
  * This also allows you to audit the software for security holes.
  *
- * Source code also allows you to port Nmap to new platforms, fix bugs, and add
- * new features. You are highly encouraged to submit your changes as a Github PR
- * or by email to the dev@nmap.org mailing list for possible incorporation into
- * the main distribution. Unless you specify otherwise, it is understood that
- * you are offering us very broad rights to use your submissions as described in
- * the Nmap Public Source License Contributor Agreement. This is important
- * because we fund the project by selling licenses with various terms, and also
- * because the inability to relicense code has caused devastating problems for
- * other Free Software projects (such as KDE and NASM).
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
  *
  * The free version of Nmap is distributed in the hope that it will be
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,7 +60,7 @@
  *
  ***************************************************************************/
 
-/* $Id: scan_engine.h 38653 2023-04-14 17:11:46Z dmiller $ */
+/* $Id: scan_engine.h 39083 2025-02-26 17:44:43Z dmiller $ */
 
 #ifndef SCAN_ENGINE_H
 #define SCAN_ENGINE_H
@@ -319,6 +320,7 @@ struct send_delay_nfo {
   unsigned int goodRespSinceDelayChanged;
   unsigned int droppedRespSinceDelayChanged;
   struct timeval last_boost; /* Most recent time of increase to delayms.  Init to creation time. */
+  int maxdelay;
 };
 
 /* To test for rate limiting, there is a delay in sending the first packet
@@ -382,7 +384,7 @@ public:
      considered a drop), but kept in the list juts in case they come
      really late.  But after probeExpireTime(), I don't waste time
      keeping them around. Give in MICROseconds */
-  unsigned long probeExpireTime(const UltraProbe *probe) const;
+  unsigned long probeExpireTime(const UltraProbe *probe, unsigned long to_us) const;
   /* Returns OK if sending a new probe to this host is OK (to avoid
      flooding). If when is non-NULL, fills it with the time that sending
      will be OK assuming no pending probes are resolved by responses
@@ -390,10 +392,10 @@ public:
      true. */
   bool sendOK(struct timeval *when) const;
 
-  /* If there are pending probe timeouts, fills in when with the time of
-     the earliest one and returns true.  Otherwise returns false and
-     puts now in when. */
-  bool nextTimeout(struct timeval *when) const;
+  /* If there are pending probe timeouts, compares the earliest one with `when`;
+     if it is earlier than `when`, replaces `when` with the time of
+     the earliest one and returns true.  Otherwise returns false. */
+  bool soonerTimeout(struct timeval *when) const;
   UltraScanInfo *USI; /* The USI which contains this HSS */
 
   /* Removes a probe from probes_outstanding, adjusts HSS and USS
@@ -525,7 +527,7 @@ struct ultra_scan_performance_vars : public scan_performance_vars {
 struct HssPredicate {
 public:
   int operator() (const HostScanStats *lhs, const HostScanStats *rhs) const;
-  static struct sockaddr_storage *ss;
+  static const struct sockaddr_storage *ss;
 };
 
 class UltraScanInfo {
@@ -555,6 +557,9 @@ public:
   bool ping_scan_arp; /* ONLY includes arp ping scan */
   bool ping_scan_nd; /* ONLY includes ND ping scan */
   bool noresp_open_scan; /* Whether no response means a port is open */
+#ifdef WIN32
+  bool has_tcp_maxrtms; /* Whether TCP_MAXRTMS socket option is available */
+#endif
 
   /* massping state. */
   /* If ping_scan is true (unless ping_scan_arp is also true), this is the set
@@ -586,7 +591,7 @@ public:
   int removeCompletedHosts();
   /* Find a HostScanStats by its IP address in the incomplete and completed
      lists.  Returns NULL if none are found. */
-  HostScanStats *findHost(struct sockaddr_storage *ss) const;
+  HostScanStats *findHost(const struct sockaddr_storage *ss) const;
 
   double getCompletionFraction() const;
 
@@ -598,7 +603,6 @@ public:
   bool incompleteHostsEmpty() const {
     return incompleteHosts.empty();
   }
-  bool numIncompleteHostsLessThan(unsigned int n) const;
 
   unsigned int numInitialHosts() const {
     return numInitialTargets;
@@ -675,17 +679,17 @@ const char *pspectype2ascii(int type);
 
 void ultrascan_port_probe_update(UltraScanInfo *USI, HostScanStats *hss,
                                  std::list<UltraProbe *>::iterator probeI,
-                                 int newstate, struct timeval *rcvdtime,
+                                 int newstate, const struct timeval *rcvdtime,
                                  bool adjust_timing_hint = true);
 
 void ultrascan_host_probe_update(UltraScanInfo *USI, HostScanStats *hss,
                                         std::list<UltraProbe *>::iterator probeI,
-                                        int newstate, struct timeval *rcvdtime,
+                                        int newstate, const struct timeval *rcvdtime,
                                         bool adjust_timing_hint = true);
 
 void ultrascan_ping_update(UltraScanInfo *USI, HostScanStats *hss,
                                   std::list<UltraProbe *>::iterator probeI,
-                                  struct timeval *rcvdtime,
+                                  const struct timeval *rcvdtime,
                                   bool adjust_timing = true);
 #endif /* SCAN_ENGINE_H */
 
