@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *
- * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
  * Project"). Nmap is also a registered trademark of the Nmap Project.
  *
  * This program is distributed under the terms of the Nmap Public Source
@@ -41,15 +41,16 @@
  * right to know exactly what a program is going to do before they run it.
  * This also allows you to audit the software for security holes.
  *
- * Source code also allows you to port Nmap to new platforms, fix bugs, and add
- * new features. You are highly encouraged to submit your changes as a Github PR
- * or by email to the dev@nmap.org mailing list for possible incorporation into
- * the main distribution. Unless you specify otherwise, it is understood that
- * you are offering us very broad rights to use your submissions as described in
- * the Nmap Public Source License Contributor Agreement. This is important
- * because we fund the project by selling licenses with various terms, and also
- * because the inability to relicense code has caused devastating problems for
- * other Free Software projects (such as KDE and NASM).
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
  *
  * The free version of Nmap is distributed in the hope that it will be
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,7 +60,7 @@
  *
  ***************************************************************************/
 
-/* $Id: targets.cc 38653 2023-04-14 17:11:46Z dmiller $ */
+/* $Id: targets.cc 39125 2025-04-16 00:00:05Z dmiller $ */
 
 
 #include <nbase.h>
@@ -73,13 +74,9 @@
 #include "nmap_dns.h"
 #include "utils.h"
 #include "nmap_error.h"
-#include "xml.h"
+#include "output.h"
 
 extern NmapOps o;
-#ifdef WIN32
-/* from libdnet's intf-win32.c */
-extern "C" int g_has_npcap_loopback;
-#endif
 
 /* Conducts an ARP ping sweep of the given hosts to determine which ones
    are up on a local ethernet network */
@@ -90,10 +87,11 @@ static void arpping(Target *hostbatch[], int num_hosts) {
   int targetno;
   targets.reserve(num_hosts);
 
+  /* Default timeout should be much lower for arp */
+  int default_to = box(o.minRttTimeout(), o.initialRttTimeout(), INITIAL_ARP_RTT_TIMEOUT) * 1000;
   for (targetno = 0; targetno < num_hosts; targetno++) {
     initialize_timeout_info(&hostbatch[targetno]->to);
-    /* Default timout should be much lower for arp */
-    hostbatch[targetno]->to.timeout = MAX(o.minRttTimeout(), MIN(o.initialRttTimeout(), INITIAL_ARP_RTT_TIMEOUT)) * 1000;
+    hostbatch[targetno]->to.timeout = default_to;
     if (!hostbatch[targetno]->SrcMACAddress()) {
       bool islocal = islocalhost(hostbatch[targetno]->TargetSockAddr());
       if (islocal) {
@@ -115,8 +113,10 @@ static void arpping(Target *hostbatch[], int num_hosts) {
   if (!targets.empty()) {
     if (targets[0]->af() == AF_INET)
       ultra_scan(targets, NULL, PING_SCAN_ARP);
-    else
+    else {
+      assert(targets[0]->af() == AF_INET6);
       ultra_scan(targets, NULL, PING_SCAN_ND);
+    }
   }
   return;
 }
@@ -133,21 +133,6 @@ void returnhost(HostGroupState *hs) {
   hs->next_batch_no--;
 }
 
-/* Is the host passed as Target to be excluded? Much of this logic had
-   to be rewritten from wam's original code to allow for the objects */
-static int hostInExclude(struct sockaddr *checksock, size_t checksocklen,
-                  const struct addrset *exclude_group) {
-  if (exclude_group == NULL)
-    return 0;
-
-  if (checksock == NULL)
-    return 0;
-
-  if (addrset_contains(exclude_group,checksock))
-    return 1;
-  return 0;
-}
-
 /* Load an exclude list from a file for --excludefile. */
 int load_exclude_file(struct addrset *excludelist, FILE *fp) {
   char host_spec[1024];
@@ -284,7 +269,7 @@ bool target_needs_new_hostgroup(Target **targets, int targets_sz, const Target *
    The target_expressions array MUST REMAIN VALID IN MEMORY as long as
    this class instance is used -- the array is NOT copied.
  */
-HostGroupState::HostGroupState(int lookahead, int rnd, int argc, const char **argv) {
+HostGroupState::HostGroupState(int lookahead, int rnd, bool gen_rand, unsigned long num_random, int argc, const char **argv) {
   assert(lookahead > 0);
   this->argc = argc;
   this->argv = argv;
@@ -295,6 +280,9 @@ HostGroupState::HostGroupState(int lookahead, int rnd, int argc, const char **ar
   current_batch_sz = 0;
   next_batch_no = 0;
   randomize = rnd;
+  if (gen_rand) {
+    current_group.generate_random_ips(num_random);
+  }
 }
 
 HostGroupState::~HostGroupState() {
@@ -314,7 +302,7 @@ void HostGroupState::undefer() {
 const char *HostGroupState::next_expression() {
   if (o.max_ips_to_scan == 0 || o.numhosts_scanned + this->current_batch_sz < o.max_ips_to_scan) {
     const char *expr;
-    expr = grab_next_host_spec(o.inputfd, o.generate_random_ips, this->argc, this->argv);
+    expr = grab_next_host_spec(o.inputfd, this->argc, this->argv);
     if (expr != NULL)
       return expr;
   }
@@ -344,18 +332,6 @@ const char *HostGroupState::next_expression() {
   return NULL;
 }
 
-/* Add a <target> element to the XML stating that a target specification was
-   ignored. This can be because of, for example, a DNS resolution failure, or a
-   syntax error. */
-static void log_bogus_target(const char *expr) {
-  xml_open_start_tag("target");
-  xml_attribute("specification", "%s", expr);
-  xml_attribute("status", "skipped");
-  xml_attribute("reason", "invalid");
-  xml_close_empty_tag();
-  xml_newline();
-}
-
 /* Returns a newly allocated Target with the given address. Handles all the
    details like setting the Target's address and next hop. */
 static Target *setup_target(const HostGroupState *hs,
@@ -398,7 +374,7 @@ static Target *setup_target(const HostGroupState *hs,
         t->setSrcMACAddress(rnfo.ii.mac);
     }
 #ifdef WIN32
-    else if (g_has_npcap_loopback && rnfo.ii.device_type == devt_loopback) {
+    else if (o.have_pcap && rnfo.ii.device_type == devt_loopback) {
       if (o.spoofMACAddress())
         t->setSrcMACAddress(o.spoofMACAddress());
       else
@@ -421,6 +397,29 @@ bail:
   return NULL;
 }
 
+bool HostGroupState::get_next_host(struct sockaddr_storage *ss, size_t *sslen, struct addrset *exclude_group) {
+  unsigned long num_queued = o.numhosts_scanned + current_batch_sz;
+  if (o.max_ips_to_scan > 0 && num_queued >= o.max_ips_to_scan) {
+    return false;
+  }
+
+  do {
+    // If the expression can't generate any more targets
+    while (current_group.get_next_host(ss, sslen) != 0) {
+      if (!current_group.load_expressions(this, o.af())) {
+        return false;
+      }
+    }
+    /* Check exclude list. */
+    if (!addrset_contains(exclude_group, (const struct sockaddr *) ss)) {
+      current_group.reject_last_host();
+      break;
+    }
+  } while (true);
+
+  return true;
+}
+
 static Target *next_target(HostGroupState *hs, struct addrset *exclude_group,
   const struct scan_lists *ports, int pingtype) {
   struct sockaddr_storage ss;
@@ -436,20 +435,9 @@ static Target *next_target(HostGroupState *hs, struct addrset *exclude_group,
 
 tryagain:
 
-  if (hs->current_group.get_next_host(&ss, &sslen) != 0) {
-    const char *expr;
-    /* We are going to have to pop in another expression. */
-    for (;;) {
-      expr = hs->next_expression();
-      if (expr == NULL)
-        /* That's the last of them. */
-        return NULL;
-      if (hs->current_group.parse_expr(expr, o.af()) == 0)
-        break;
-      else
-        log_bogus_target(expr);
-    }
-    goto tryagain;
+  if (!hs->get_next_host(&ss, &sslen, exclude_group)) {
+    /* That's the last of them. */
+    return NULL;
   }
 
   assert(ss.ss_family == o.af());
@@ -463,10 +451,6 @@ tryagain:
     goto tryagain;
   }
 
-  /* Check exclude list. */
-  if (hostInExclude((struct sockaddr *) &ss, sslen, exclude_group))
-    goto tryagain;
-
   t = setup_target(hs, &ss, sslen, pingtype);
   if (t == NULL)
     goto tryagain;
@@ -517,19 +501,9 @@ static void refresh_hostbatch(HostGroupState *hs, struct addrset *exclude_group,
   /* First I'll do the ARP ping if all of the machines in the group are
      directly connected over ethernet.  I may need the MAC addresses
      later anyway. */
-  if (hs->hostbatch[0]->ifType() == devt_ethernet &&
-      hs->hostbatch[0]->af() == AF_INET &&
-      hs->hostbatch[0]->directlyConnected() &&
-      o.sendpref != PACKET_SEND_IP_STRONG &&
-      o.implicitARPPing) {
-    arpping(hs->hostbatch, hs->current_batch_sz);
-    arpping_done = true;
-  }
-
   /* No other interface types are supported by ND ping except devt_ethernet
      at the moment. */
   if (hs->hostbatch[0]->ifType() == devt_ethernet &&
-      hs->hostbatch[0]->af() == AF_INET6 &&
       hs->hostbatch[0]->directlyConnected() &&
       o.sendpref != PACKET_SEND_IP_STRONG &&
       o.implicitARPPing) {
@@ -547,6 +521,7 @@ static void refresh_hostbatch(HostGroupState *hs, struct addrset *exclude_group,
           error("%s: Failed to determine dst MAC address for target %s",
               __func__, hs->hostbatch[i]->NameIP());
           hs->hostbatch[i]->flags = HOST_DOWN;
+          hs->hostbatch[i]->reason.reason_id = ER_NOROUTE;
         }
       }
     }
