@@ -5,134 +5,66 @@
  * libpcap, etc.                                                           *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: tcpip.cc 36788 2017-06-07 12:32:38Z dmiller $ */
+/* $Id: tcpip.cc 39118 2025-04-11 21:45:17Z dmiller $ */
 
 #include "nmap.h"
 
+#include <locale.h>
 #include "nbase.h"
 #include <dnet.h>
 #include "tcpip.h"
@@ -163,11 +95,15 @@ static PacketCounter PktCt;
    AF_INET6, depending on the operating system. */
 int nmap_raw_socket() {
   int rawsd;
+  int one = 1;
 
   rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
   if (rawsd < 0)
     return rawsd;
-  broadcast_socket(rawsd);
+  if (setsockopt (rawsd, SOL_SOCKET, SO_BROADCAST, (const char *) &one, sizeof(int)) != 0) {
+    error("Failed to secure socket broadcasting permission");
+    perror("setsockopt");
+  }
 #ifndef WIN32
   sethdrinclude(rawsd);
 #endif
@@ -255,9 +191,9 @@ void PacketTrace::traceArp(pdirection pdir, const u8 *frame, u32 len,
 void PacketTrace::traceND(pdirection pdir, const u8 *frame, u32 len,
                           struct timeval *now) {
   struct timeval tv;
-  struct ip6_hdr *ip6;
-  struct icmpv6_hdr *icmpv6;
-  union icmpv6_msg *msg;
+  const struct ip6_hdr *ip6;
+  const struct icmpv6_hdr *icmpv6;
+  const union icmpv6_msg *msg;
   size_t msg_len;
   const char *label;
   char src[INET6_ADDRSTRLEN], dst[INET6_ADDRSTRLEN];
@@ -296,7 +232,7 @@ void PacketTrace::traceND(pdirection pdir, const u8 *frame, u32 len,
     if (msg_len < 20) {
       Snprintf(desc, sizeof(desc), "packet too short");
     } else {
-      inet_ntop(AF_INET6, &msg->nd.icmpv6_target, who_has, sizeof(who_has));
+      inet_ntop(AF_INET6, (void *)&msg->nd.icmpv6_target, who_has, sizeof(who_has));
       Snprintf(desc, sizeof(desc), "who has %s", who_has);
     }
   } else if (icmpv6->icmpv6_type == ICMPV6_NEIGHBOR_ADVERTISEMENT) {
@@ -307,7 +243,7 @@ void PacketTrace::traceND(pdirection pdir, const u8 *frame, u32 len,
       /* We only handle target link-layer address in the first option. */
       Snprintf(desc, sizeof(desc), "no link-layer address");
     } else {
-      inet_ntop(AF_INET6, &msg->nd.icmpv6_target, tgt_is, sizeof(tgt_is));
+      inet_ntop(AF_INET6, (void *)&msg->nd.icmpv6_target, tgt_is, sizeof(tgt_is));
       Snprintf(desc, sizeof(desc), "%s is at %s",
                tgt_is, eth_ntoa(&msg->nd.icmpv6_mac));
     }
@@ -316,8 +252,8 @@ void PacketTrace::traceND(pdirection pdir, const u8 *frame, u32 len,
     return;
   }
 
-  inet_ntop(AF_INET6, &ip6->ip6_src, src, sizeof(src));
-  inet_ntop(AF_INET6, &ip6->ip6_dst, dst, sizeof(dst));
+  inet_ntop(AF_INET6, (void *)&ip6->ip6_src, src, sizeof(src));
+  inet_ntop(AF_INET6, (void *)&ip6->ip6_dst, dst, sizeof(dst));
   log_write(LOG_STDOUT | LOG_NORMAL, "%s (%.4fs) %s %s > %s %s\n",
             (pdir == SENT) ? "SENT" : "RCVD",
             o.TimeSinceStart(&tv), label, src, dst, desc);
@@ -390,9 +326,9 @@ void PacketTrace::traceConnect(u8 proto, const struct sockaddr *sock,
                                int socklen, int connectrc,
                                int connect_errno,
                                const struct timeval *now) {
-  struct sockaddr_in *sin = (struct sockaddr_in *) sock;
+  const struct sockaddr_in *sin = (struct sockaddr_in *) sock;
 #if HAVE_IPV6
-  struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) sock;
+  const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) sock;
 #endif
   struct timeval tv;
   char errbuf[64] = "";
@@ -450,11 +386,11 @@ void PacketTrace::traceConnect(u8 proto, const struct sockaddr *sock,
 /* Converts an IP address given in a sockaddr_storage to an IPv4 or
    IPv6 IP address string.  Since a static buffer is returned, this is
    not thread-safe and can only be used once in calls like printf() */
-const char *inet_socktop(struct sockaddr_storage *ss) {
+const char *inet_socktop(const struct sockaddr_storage *ss) {
   static char buf[INET6_ADDRSTRLEN];
-  struct sockaddr_in *sin = (struct sockaddr_in *) ss;
+  const struct sockaddr_in *sin = (struct sockaddr_in *) ss;
 #if HAVE_IPV6
-  struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) ss;
+  const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) ss;
 #endif
 
   if (inet_ntop(sin->sin_family, (sin->sin_family == AF_INET) ?
@@ -483,9 +419,20 @@ struct addrinfo *resolve_all(const char *hostname, int pf) {
   hints.ai_family = pf;
   /* Otherwise we get multiple identical addresses with different socktypes. */
   hints.ai_socktype = SOCK_DGRAM;
+#ifdef AI_IDN
+  /* Try resolving internationalized domain names */
+  hints.ai_flags = AI_IDN;
+  setlocale(LC_CTYPE, "");
+#endif
   rc = getaddrinfo(hostname, NULL, &hints, &result);
-  if (rc != 0)
+#ifdef AI_IDN
+  setlocale(LC_CTYPE, o.locale);
+#endif
+  if (rc != 0){
+    if (o.debugging > 1)
+      error("Error resolving %s: %s", hostname, gai_strerror(rc));
     return NULL;
+  }
 
   return result;
 }
@@ -496,7 +443,7 @@ struct addrinfo *resolve_all(const char *hostname, int pf) {
 static int send_ipv4_packet(int sd, const struct eth_nfo *eth,
                             const struct sockaddr_in *dst,
                             const u8 *packet, unsigned int packetlen) {
-  struct ip *ip = (struct ip *) packet;
+  const struct ip *ip = (struct ip *) packet;
   int res;
 
   assert(packet);
@@ -530,7 +477,7 @@ static int send_ipv6_packet(int sd, const struct eth_nfo *eth,
 int send_ip_packet(int sd, const struct eth_nfo *eth,
                    const struct sockaddr_storage *dst,
                    const u8 *packet, unsigned int packetlen) {
-  struct ip *ip = (struct ip *) packet;
+  const struct ip *ip = (struct ip *) packet;
 
   /* Ensure there's enough to read ip->ip_v at least. */
   if (packetlen < 1)
@@ -1220,8 +1167,8 @@ u8 *build_igmp_raw(const struct in_addr *source,
    of a TCP packet*/
 int readtcppacket(const u8 *packet, int readdata) {
 
-  struct ip *ip = (struct ip *) packet;
-  struct tcp_hdr *tcp = (struct tcp_hdr *) (packet + sizeof(struct ip));
+  const struct ip *ip = (struct ip *) packet;
+  const struct tcp_hdr *tcp = (struct tcp_hdr *) (packet + sizeof(struct ip));
   const unsigned char *data = packet + sizeof(struct ip) + sizeof(struct tcp_hdr);
   int tot_len;
   struct in_addr bullshit, bullshit2;
@@ -1296,8 +1243,8 @@ int readtcppacket(const u8 *packet, int readdata) {
 /* A simple function I wrote to help in debugging, shows the important fields
    of a UDP packet*/
 int readudppacket(const u8 *packet, int readdata) {
-  struct ip *ip = (struct ip *) packet;
-  struct udp_hdr *udp = (struct udp_hdr *) (packet + sizeof(struct ip));
+  const struct ip *ip = (struct ip *) packet;
+  const struct udp_hdr *udp = (struct udp_hdr *) (packet + sizeof(struct ip));
   const unsigned char *data = packet + sizeof(struct ip) + sizeof(struct udp_hdr);
   int tot_len;
   struct in_addr bullshit, bullshit2;
@@ -1343,8 +1290,8 @@ int readudppacket(const u8 *packet, int readdata) {
 
 /* Used by validatepkt() to validate the TCP header (including option lengths).
    The options checked are MSS, WScale, SackOK, Sack, and Timestamp. */
-static bool validateTCPhdr(u8 *tcpc, unsigned len) {
-  struct tcp_hdr *tcp = (struct tcp_hdr *) tcpc;
+static bool validateTCPhdr(const u8 *tcpc, unsigned len) {
+  const struct tcp_hdr *tcp = (struct tcp_hdr *) tcpc;
   unsigned hdrlen, optlen;
 
   hdrlen = tcp->th_off * 4;
@@ -1357,56 +1304,61 @@ static bool validateTCPhdr(u8 *tcpc, unsigned len) {
   tcpc += sizeof(struct tcp_hdr);
   optlen = hdrlen - sizeof(struct tcp_hdr);
 
-  while (optlen > 0) {
+// This macro guarantees optlen does not underflow by returning if optlen < expected
+#define OPTLEN_IS(expected) do { \
+  if ((expected) == 0 || optlen < (expected) || hdrlen != (expected)) \
+    return false; \
+  optlen -= (expected); \
+  tcpc += (expected); \
+} while(0);
+
+  while (optlen > 1) {
+    hdrlen = *(tcpc + 1);
     switch (*tcpc) {
+    case 0: // EOL
+      /* Options processing is over. */
+      return true;
+    case 1: // NOP
+      /* 1 byte, no length. All other options have a length. */
+      optlen--;
+      tcpc++;
+      break;
     case 2: /* MSS */
-      if (optlen < 4)
-        return false;
-      optlen -= 4;
-      tcpc += 4;
+      OPTLEN_IS(4);
       break;
     case 3: /* Window Scale */
-      if (optlen < 3)
-        return false;
-      optlen -= 3;
-      tcpc += 3;
+      OPTLEN_IS(3);
       break;
     case 4: /* SACK Permitted */
-      if (optlen < 2)
-        return false;
-      optlen -= 2;
-      tcpc += 2;
+      OPTLEN_IS(2);
       break;
     case 5: /* SACK */
-      if (optlen < *++tcpc)
+      if (!(hdrlen - 2) || ((hdrlen - 2) % 8))
         return false;
-      if (!(*tcpc - 2) || ((*tcpc - 2) % 8))
-        return false;
-      optlen -= *tcpc;
-      tcpc += (*tcpc - 1);
+      OPTLEN_IS(hdrlen);
       break;
     case 8: /* Timestamp */
-      if (optlen < 10)
-        return false;
-      optlen -= 10;
-      tcpc += 10;
+      OPTLEN_IS(10);
       break;
     case 14: /* Alternate checksum */
       /* Sometimes used for hardware checksum offloading
        * ftp://ftp.ucsd.edu/pub/csl/fastnet/faq.txt
        */
-      if (optlen < 3)
-        return false;
-      optlen -= 3;
-      tcpc += 3;
+      OPTLEN_IS(3);
       break;
     default:
-      optlen--;
-      tcpc++;
+      OPTLEN_IS(hdrlen);
       break;
     }
   }
 
+  if (optlen == 1) {
+    // Only 1 byte left in options, this has to be NOP or EOL
+    return (*tcpc == 0 || *tcpc == 1);
+  }
+  // There is no way out of the previous loop that does not satisfy optlen == 0 or optlen == 1
+  assert(optlen == 0);
+
   return true;
 }
 
@@ -1426,8 +1378,8 @@ static bool validateTCPhdr(u8 *tcpc, unsigned len) {
  * read more than the IP header says we should have so as to not pass garbage
  * data to the caller.
  */
-static bool validatepkt(u8 *ipc, unsigned *len) {
-  struct ip *ip = (struct ip *) ipc;
+static bool validatepkt(const u8 *ipc, unsigned *len) {
+  const struct ip *ip = (struct ip *) ipc;
   const void *data;
   unsigned int datalen, iplen;
   u8 hdr;
@@ -1526,14 +1478,14 @@ static bool validatepkt(u8 *ipc, unsigned *len) {
    linknfo->header will be filled with the appropriate values. */
 /* Specifying true for validate will enable validity checks against the
    received IP packet.  See validatepkt() for a list of checks. */
-char *readipv4_pcap(pcap_t *pd, unsigned int *len, long to_usec,
+const u8 *readipv4_pcap(pcap_t *pd, unsigned int *len, long to_usec,
                     struct timeval *rcvdtime, struct link_header *linknfo,
                     bool validate) {
-  char *buf;
+  const u8 *buf;
 
   buf = readip_pcap(pd, len, to_usec, rcvdtime, linknfo, validate);
   if (buf != NULL) {
-    struct ip *ip;
+    const struct ip *ip;
 
     ip = (struct ip *) buf;
     if (*len < 1 || ip->ip_v != 4)
@@ -1543,304 +1495,75 @@ char *readipv4_pcap(pcap_t *pd, unsigned int *len, long to_usec,
   return buf;
 }
 
-char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec,
+static bool accept_any (const unsigned char *p, const struct pcap_pkthdr *h, int datalink, size_t offset) {
+  return true;
+}
+
+static bool accept_ip (const unsigned char *p, const struct pcap_pkthdr *h, int datalink, size_t offset) {
+  const struct ip *ip = NULL;
+
+  if (h->caplen < offset + sizeof(struct ip)) {
+    return false;
+  }
+  ip = (struct ip *) (p + offset);
+  switch (ip->ip_v) {
+    case 4:
+    case 6:
+      break;
+    default:
+      return false;
+      break;
+  }
+
+  return true;
+}
+
+const u8 *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec,
                   struct timeval *rcvdtime, struct link_header *linknfo, bool validate) {
-  unsigned int offset = 0;
-  struct pcap_pkthdr head;
-  char *p;
   int datalink;
-  int timedout = 0;
-  struct timeval tv_start, tv_end;
-  static char *alignedbuf = NULL;
-  static unsigned int alignedbufsz = 0;
-  static int warning = 0;
+  size_t offset = 0;
+  struct pcap_pkthdr *head;
+  const u8 *p;
+  int got_one = 0;
 
   if (linknfo) {
     memset(linknfo, 0, sizeof(*linknfo));
   }
 
-  if (!pd)
-    fatal("NULL packet device passed to %s", __func__);
-
-  if (to_usec < 0) {
-    if (!warning) {
-      warning = 1;
-      error("WARNING: Negative timeout value (%lu) passed to %s() -- using 0", to_usec, __func__);
-    }
-    to_usec = 0;
+  if (validate) {
+    got_one = read_reply_pcap(pd, to_usec, accept_ip, &p, &head, rcvdtime, &datalink, &offset);
+  }
+  else {
+    got_one = read_reply_pcap(pd, to_usec, accept_any, &p, &head, rcvdtime, &datalink, &offset);
   }
 
-  /* New packet capture device, need to recompute offset */
-  if ((datalink = pcap_datalink(pd)) < 0)
-    fatal("Cannot obtain datalink information: %s", pcap_geterr(pd));
-
-  /* NOTE: IF A NEW OFFSET EVER EXCEEDS THE CURRENT MAX (24), ADJUST
-     MAX_LINK_HEADERSZ in libnetutil/netutil.h */
-  switch (datalink) {
-  case DLT_EN10MB:
-    offset = 14;
-    break;
-  case DLT_IEEE802:
-    offset = 22;
-    break;
-#ifdef __amigaos__
-  case DLT_MIAMI:
-    offset = 16;
-    break;
-#endif
-#ifdef DLT_LOOP
-  case DLT_LOOP:
-#endif
-  case DLT_NULL:
-    offset = 4;
-    break;
-  case DLT_SLIP:
-#ifdef DLT_SLIP_BSDOS
-  case DLT_SLIP_BSDOS:
-#endif
-#if (FREEBSD || OPENBSD || NETBSD || BSDI || MACOSX)
-    offset = 16;
-#else
-    offset = 24; /* Anyone use this??? */
-#endif
-    break;
-  case DLT_PPP:
-#ifdef DLT_PPP_BSDOS
-  case DLT_PPP_BSDOS:
-#endif
-#ifdef DLT_PPP_SERIAL
-  case DLT_PPP_SERIAL:
-#endif
-#ifdef DLT_PPP_ETHER
-  case DLT_PPP_ETHER:
-#endif
-#if (FREEBSD || OPENBSD || NETBSD || BSDI || MACOSX)
-    offset = 4;
-#else
-#ifdef SOLARIS
-    offset = 8;
-#else
-    offset = 24; /* Anyone use this? */
-#endif /* ifdef solaris */
-#endif /* if freebsd || openbsd || netbsd || bsdi */
-    break;
-  case DLT_RAW:
-    offset = 0;
-    break;
-  case DLT_FDDI:
-    offset = 21;
-    break;
-#ifdef DLT_ENC
-  case DLT_ENC:
-    offset = 12;
-    break;
-#endif /* DLT_ENC */
-#ifdef DLT_LINUX_SLL
-  case DLT_LINUX_SLL:
-    offset = 16;
-    break;
-#endif
-#ifdef DLT_IPNET
-  case DLT_IPNET:
-    offset = 24;
-    break;
-#endif
-  default:
-    p = (char *) pcap_next(pd, &head);
-    if (head.caplen == 0) {
-      /* Lets sleep a brief time and try again to increase the chance of seeing
-         a real packet ... */
-      usleep(500000);
-      p = (char *) pcap_next(pd, &head);
-    }
-    if (head.caplen > 100000) {
-      fatal("FATAL: %s: bogus caplen from libpcap (%d) on interface type %d", __func__, head.caplen, datalink);
-    }
-    error("FATAL:  Unknown datalink type (%d). Caplen: %d; Packet:", datalink, head.caplen);
-    nmap_hexdump((unsigned char *) p, head.caplen);
-    exit(1);
-  }
-
-  if (to_usec > 0) {
-    gettimeofday(&tv_start, NULL);
-  }
-
-  do {
-
-    p = NULL;
-    /* It may be that protecting this with !pcap_selectable_fd_one_to_one is not
-       necessary, that it is always safe to do a nonblocking read in this way on
-       all platforms. But I have only tested it on Solaris. */
-    if (!pcap_selectable_fd_one_to_one()) {
-      int rc, nonblock;
-
-      nonblock = pcap_getnonblock(pd, NULL);
-      assert(nonblock == 0);
-      rc = pcap_setnonblock(pd, 1, NULL);
-      assert(rc == 0);
-      p = (char *) pcap_next(pd, &head);
-      rc = pcap_setnonblock(pd, nonblock, NULL);
-      assert(rc == 0);
-    }
-
-    if (p == NULL) {
-      /* Nonblocking pcap_next didn't get anything. */
-      if (pcap_select(pd, to_usec) == 0)
-        timedout = 1;
-      else
-        p = (char *) pcap_next(pd, &head);
-    }
-
-    if (p) {
-      if (head.caplen <= offset) {
-        *len = 0;
-        return NULL;
-      }
-      if (offset && linknfo) {
-        linknfo->datalinktype = datalink;
-        linknfo->headerlen = offset;
-        assert(offset <= MAX_LINK_HEADERSZ);
-        memcpy(linknfo->header, p, MIN(sizeof(linknfo->header), offset));
-      }
-      p += offset;
-    }
-    if (!p) {
-      /* Should we timeout? */
-      if (to_usec == 0) {
-        timedout = 1;
-      } else if (to_usec > 0) {
-        gettimeofday(&tv_end, NULL);
-        if (TIMEVAL_SUBTRACT(tv_end, tv_start) >= to_usec) {
-          timedout = 1;
-        }
-      }
-    }
-  } while (!timedout && (!p));
-
-  if (timedout) {
+  if (!got_one) {
     *len = 0;
     return NULL;
   }
-  *len = head.caplen - offset;
-  if (*len > alignedbufsz) {
-    alignedbuf = (char *) safe_realloc(alignedbuf, *len);
-    alignedbufsz = *len;
-  }
-  memcpy(alignedbuf, p, *len);
+
+  *len = head->caplen - offset;
+  p += offset;
 
   if (validate) {
-    /* Let's see if this packet passes inspection.. */
-    if (!validatepkt((u8 *) alignedbuf, len)) {
+    if (!validatepkt(p, len)) {
       *len = 0;
       return NULL;
     }
   }
-  // printf("Just got a packet at %li,%li\n", head.ts.tv_sec, head.ts.tv_usec);
-  if (rcvdtime) {
-    // FIXME: I eventually need to figure out why Windows head.ts time is sometimes BEFORE the time I
-    // sent the packet (which is according to gettimeofday() in nbase).  For now, I will sadly have to
-    // use gettimeofday() for Windows in this case
-    // Actually I now allow .05 discrepancy.   So maybe this isn't needed.  I'll comment out for now.
-    // Nope: it is still needed at least for Windows.  Sometimes the time from he pcap header is a
-    // COUPLE SECONDS before the gettimeofday() results :(.
-#if defined(WIN32) || defined(__amigaos__)
-    gettimeofday(&tv_end, NULL);
-    *rcvdtime = tv_end;
-#else
-    rcvdtime->tv_sec = head.ts.tv_sec;
-    rcvdtime->tv_usec = head.ts.tv_usec;
-    assert(head.ts.tv_sec);
-#endif
+  if (offset && linknfo) {
+    linknfo->datalinktype = datalink;
+    linknfo->headerlen = offset;
+    linknfo->header = p;
   }
-
   if (rcvdtime)
-    PacketTrace::trace(PacketTrace::RCVD, (u8 *) alignedbuf, *len,
-                       rcvdtime);
+    PacketTrace::trace(PacketTrace::RCVD, (u8 *) p, *len,
+        rcvdtime);
   else
-    PacketTrace::trace(PacketTrace::RCVD, (u8 *) alignedbuf, *len);
+    PacketTrace::trace(PacketTrace::RCVD, (u8 *) p, *len);
 
-  return alignedbuf;
-}
-
-/* Attempts to read one IPv6 Neighbor Solicitation reply packet from the pcap
-   descriptor pd.  If it receives one, fills in sendermac (must pass
-   in 6 bytes), senderIP, and rcvdtime (can be NULL if you don't care)
-   and returns 1.  If it times out and reads no Neighbor Advertisement, returns
-   0.  to_usec is the timeout period in microseconds.  Use 0 to avoid
-   blocking to the extent possible.  Returns -1 or exits if there is
-   an error.  The last parameter is a pointer to a callback function
-   that can be used for packet tracing. This is intended to be used
-   by Nmap only. Any other calling this should pass NULL instead. */
-int read_na_pcap(pcap_t *pd, u8 *sendermac, struct sockaddr_in6 *senderIP, long to_usec,
-                 struct timeval *rcvdtime, bool *has_mac) {
-  struct ip *ip_tmp;
-  struct icmpv6_hdr *icmp6_header;
-  struct icmpv6_msg_nd *na;
-  struct timeval tv_start, tv_end;
-  const void *data = NULL;
-  unsigned int datalen;
-  static int warning = 0;
-  int timedout = 0;
-  struct abstract_ip_hdr hdr;
-  struct link_header linknfo;
-
-
-  if (to_usec < 0) {
-    if (!warning) {
-      warning = 1;
-      error("WARNING: Negative timeout value (%lu) passed to %s() -- using 0", to_usec, __func__);
-    }
-    to_usec = 0;
-  }
-
-  if (to_usec > 0) {
-    gettimeofday(&tv_start, NULL);
-  }
-
-  do {
-    ip_tmp = (struct ip *) readip_pcap(pd, &datalen, to_usec, rcvdtime, &linknfo, true);
-    if (ip_tmp) { //Check Neighbor Advertisement Packet.
-      /* OK, we got a packet.  Most packet validity tests are taken care
-       * of in readip_pcap, so this is simple
-       */
-      data = ip_get_data(ip_tmp, &datalen, &hdr);
-      if (data == NULL)
-        continue;
-      if (hdr.proto == IPPROTO_ICMPV6) {
-        icmp6_header = (struct icmpv6_hdr *)data;
-        na = (struct icmpv6_msg_nd *) ((unsigned char*)data + ICMPV6_HDR_LEN);
-        if (icmp6_header->icmpv6_type == ICMPV6_NEIGHBOR_ADVERTISEMENT &&
-            icmp6_header->icmpv6_code == 0) {
-          //Set target IPv6 address
-          senderIP->sin6_family = AF_INET6;
-          memcpy(&senderIP->sin6_addr.s6_addr, &na->icmpv6_target, 16);
-          //Set MAC
-          if (datalen == ICMPV6_HDR_LEN + sizeof(struct icmpv6_msg_nd)) {
-            if (na->icmpv6_option_type == 2 && na->icmpv6_option_length == 1) {
-              *has_mac = true;
-              memcpy(sendermac, &na->icmpv6_mac, 6);
-            }
-          } else {
-            *has_mac = false;
-          }
-        }
-      }
-    } else {
-      /* Should we timeout? */
-      if (to_usec == 0) {
-        timedout = 1;
-      } else if (to_usec > 0) {
-        gettimeofday(&tv_end, NULL);
-        if (TIMEVAL_SUBTRACT(tv_end, tv_start) >= to_usec) {
-          timedout = 1;
-        }
-      }
-    }
-  } while (!timedout && !ip_tmp);
-
-  if (timedout)
-    return 0;
-  return 1;
+  *len = head->caplen - offset;
+  return p;
 }
 
 // Returns whether the packet receive time value obtained from libpcap
@@ -1883,21 +1606,16 @@ void pcap_print_stats(int logt, pcap_t *pd) {
    This function returns 0 if it ends up setting the MAC, nonzero otherwise. */
 int setTargetMACIfAvailable(Target *target, struct link_header *linkhdr,
                             const struct sockaddr_storage *src, int overwrite) {
-  struct sockaddr_storage addr;
-  size_t addr_len;
-
   if (!linkhdr || !target || !src)
     return 1;
 
-  if (linkhdr->datalinktype != DLT_EN10MB || linkhdr->headerlen != 14)
+  if (linkhdr->datalinktype != DLT_EN10MB || linkhdr->headerlen < 14)
     return 2;
 
   if (!overwrite && target->MACAddress())
     return 3;
 
-  addr_len = sizeof(addr);
-  target->TargetSockAddr(&addr, &addr_len);
-  if (sockaddr_storage_cmp(src, &addr) != 0)
+  if (sockaddr_storage_cmp(src, target->TargetSockAddr()) != 0)
     return 4;
 
   /* Sometimes bogus MAC address still gets through, like during some localhost scans */
@@ -1997,29 +1715,6 @@ bool getNextHopMAC(const char *iface, const u8 *srcmac, const struct sockaddr_st
 }
 
 
-
-
-
-
-
-
-pcap_if_t *getpcapinterfaces() {
-#ifndef WIN32
-  return NULL;
-#endif
-  pcap_if_t *p_ifaces;
-  char errbuf[PCAP_ERRBUF_SIZE];
-
-  if ((pcap_findalldevs(&p_ifaces, errbuf)) == -1) {
-    fatal("pcap_findalldevs(): Cannot retrieve pcap interfaces: %s", errbuf);
-    return NULL;
-  }
-  return p_ifaces;
-}
-
-
-
-
 int nmap_route_dst(const struct sockaddr_storage *dst, struct route_nfo *rnfo) {
   struct sockaddr_storage spoofss;
   size_t spoofsslen;
@@ -2050,19 +1745,6 @@ void max_rcvbuf(int sd) {
 }
 
 
-/* Give broadcast permission to a socket */
-void broadcast_socket(int sd) {
-  int one = 1;
-#ifdef WIN32
-  if (sd == 501)
-    return;
-#endif
-  if (setsockopt (sd, SOL_SOCKET, SO_BROADCAST, (const char *) &one, sizeof(int)) != 0) {
-    error("Failed to secure socket broadcasting permission");
-    perror("setsockopt");
-  }
-}
-
 /* Do a receive (recv()) on a socket and stick the results (up to
    len) into buf .  Give up after 'seconds'.  Returns the number of
    bytes read (or -1 in the case of an error.  It only does one recv
@@ -2078,7 +1760,7 @@ int recvtime(int sd, char *buf, int len, int seconds, int *timedout) {
   timeout.tv_sec = seconds;
   timeout.tv_usec = 0;
   FD_ZERO(&readfd);
-  FD_SET(sd, &readfd);
+  checked_fd_set(sd, &readfd);
   if (timedout)
     *timedout = 0;
   res = select(sd + 1, &readfd, NULL, NULL, &timeout);
@@ -2107,7 +1789,7 @@ int recvtime(int sd, char *buf, int len, int seconds, int *timedout) {
    parameters (if non-null) are filled with 0.  Remember that the
    correct way to check for errors is to look at the return value
    since a zero ts or echots could possibly be valid. */
-int gettcpopt_ts(struct tcp_hdr *tcp, u32 *timestamp, u32 *echots) {
+int gettcpopt_ts(const struct tcp_hdr *tcp, u32 *timestamp, u32 *echots) {
 
   unsigned char *p;
   int len = 0;
