@@ -1,130 +1,69 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
-# *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
-# *                                                                         *
-# * Note that the GPL places important restrictions on "derivative works",  *
-# * yet it does not provide a detailed definition of that term.  To avoid   *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * derivative work for the purpose of this license if it does any of the   *
-# * following with any software or content covered by this license          *
-# * ("Covered Software"):                                                   *
-# *                                                                         *
-# * o Integrates source code from Covered Software.                         *
-# *                                                                         *
-# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
-# * or nmap-service-probes.                                                 *
-# *                                                                         *
-# * o Is designed specifically to execute Covered Software and parse the    *
-# * results (as opposed to typical shell or execution-menu apps, which will *
-# * execute anything you tell them to).                                     *
-# *                                                                         *
-# * o Includes Covered Software in a proprietary executable installer.  The *
-# * installers produced by InstallShield are an example of this.  Including *
-# * Nmap with other software in compressed or archival form does not        *
-# * trigger this provision, provided appropriate open source decompression  *
-# * or de-archiving software is widely available for no charge.  For the    *
-# * purposes of this license, an installer is considered to include Covered *
-# * Software even if it actually retrieves a copy of Covered Software from  *
-# * another source during runtime (such as by downloading it from the       *
-# * Internet).                                                              *
-# *                                                                         *
-# * o Links (statically or dynamically) to a library which does any of the  *
-# * above.                                                                  *
-# *                                                                         *
-# * o Executes a helper program, module, or script to do any of the above.  *
-# *                                                                         *
-# * This list is not exclusive, but is meant to clarify our interpretation  *
-# * of derived works with some common examples.  Other people may interpret *
-# * the plain GPL differently, so we consider this a special exception to   *
-# * the GPL that we apply to Covered Software.  Works which meet any of     *
-# * these conditions must conform to all of the terms of this license,      *
-# * particularly including the GPL Section 3 requirements of providing      *
-# * source code and allowing free redistribution of the work as a whole.    *
-# *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two.                                  *
-# *                                                                         *
-# * Any redistribution of Covered Software, including any derived works,    *
-# * must obey and carry forward all of the terms of this license, including *
-# * obeying all GPL rules and restrictions.  For example, source code of    *
-# * the whole work must be provided and free redistribution must be         *
-# * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the terms and conditions of this license text as well.        *
-# *                                                                         *
-# * Because this license imposes special exceptions to the GPL, Covered     *
-# * Work may not be combined (even as part of a larger work) with plain GPL *
-# * software.  The terms, conditions, and exceptions of this license must   *
-# * be included as well.  This license is incompatible with some other open *
-# * source licenses as well.  In some cases we can relicense portions of    *
-# * Nmap or grant special permissions to use it in other open source        *
-# * software.  Please contact fyodor@nmap.org with any such requests.       *
-# * Similarly, we don't incorporate incompatible open source software into  *
-# * Covered Software without special permission from the copyright holders. *
-# *                                                                         *
-# * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
-# * information.                                                            *
-# *                                                                         *
-# * If you have received a written license agreement or contract for        *
-# * Covered Software stating terms other than these, you may choose to use  *
-# * and redistribute Covered Software under those terms instead of these.   *
-# *                                                                         *
-# * Source is provided to this software because we believe users have a     *
-# * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes.          *
-# *                                                                         *
-# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to the dev@nmap.org mailing list for possible incorporation into the    *
-# * main distribution.  By sending these changes to Fyodor or one of the    *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
-# *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
-# * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
-# *                                                                         *
+# *
+# * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
+# * Project"). Nmap is also a registered trademark of the Nmap Project.
+# *
+# * This program is distributed under the terms of the Nmap Public Source
+# * License (NPSL). The exact license text applying to a particular Nmap
+# * release or source code control revision is contained in the LICENSE
+# * file distributed with that version of Nmap or source code control
+# * revision. More Nmap copyright/legal information is available from
+# * https://nmap.org/book/man-legal.html, and further information on the
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This
+# * header summarizes some key points from the Nmap license, but is no
+# * substitute for the actual license text.
+# *
+# * Nmap is generally free for end users to download and use themselves,
+# * including commercial use. It is available from https://nmap.org.
+# *
+# * The Nmap license generally prohibits companies from using and
+# * redistributing Nmap in commercial products, but we sell a special Nmap
+# * OEM Edition with a more permissive license and special features for
+# * this purpose. See https://nmap.org/oem/
+# *
+# * If you have received a written Nmap license agreement or contract
+# * stating terms other than these (such as an Nmap OEM license), you may
+# * choose to use and redistribute Nmap under those terms instead.
+# *
+# * The official Nmap Windows builds include the Npcap software
+# * (https://npcap.com) for packet capture and transmission. It is under
+# * separate license terms which forbid redistribution without special
+# * permission. So the official Nmap Windows builds may not be redistributed
+# * without special permission (such as an Nmap OEM license).
+# *
+# * Source is provided to this software because we believe users have a
+# * right to know exactly what a program is going to do before they run it.
+# * This also allows you to audit the software for security holes.
+# *
+# * Source code also allows you to port Nmap to new platforms, fix bugs, and
+# * add new features. You are highly encouraged to submit your changes as a
+# * Github PR or by email to the dev@nmap.org mailing list for possible
+# * incorporation into the main distribution. Unless you specify otherwise, it
+# * is understood that you are offering us very broad rights to use your
+# * submissions as described in the Nmap Public Source License Contributor
+# * Agreement. This is important because we fund the project by selling licenses
+# * with various terms, and also because the inability to relicense code has
+# * caused devastating problems for other Free Software projects (such as KDE
+# * and NASM).
+# *
+# * The free version of Nmap is distributed in the hope that it will be
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+# * indemnification and commercial support are all available through the
+# * Npcap OEM program--see https://nmap.org/oem/
+# *
 # ***************************************************************************/
 
-import gtk
+import gi
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk
 
 import sys
 import os
-from os.path import split, isfile, join, abspath, exists
-import errno
+from os.path import split, isfile, join, abspath
 
 # Prevent loading PyXML
 import xml
@@ -132,16 +71,15 @@ xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
 
 import xml.sax.saxutils
 
-from time import time
-
 from zenmapGUI.higwidgets.higwindows import HIGMainWindow
 from zenmapGUI.higwidgets.higdialogs import HIGDialog, HIGAlertDialog
 from zenmapGUI.higwidgets.higlabels import HIGEntryLabel
 from zenmapGUI.higwidgets.higboxes import HIGHBox, HIGVBox
-from zenmapGUI.higwidgets.higlabels import HIGSectionLabel
 
 import zenmapGUI.App
-from zenmapGUI.FileChoosers import *
+from zenmapGUI.FileChoosers import RESPONSE_OPEN_DIRECTORY, \
+        ResultsFileChooserDialog, SaveResultsFileChooserDialog, \
+        SaveToDirectoryChooserDialog
 from zenmapGUI.ScanInterface import ScanInterface
 from zenmapGUI.ProfileEditor import ProfileEditor
 from zenmapGUI.About import About
@@ -150,14 +88,12 @@ from zenmapGUI.SearchWindow import SearchWindow
 from zenmapGUI.BugReport import BugReport
 
 from zenmapCore.Name import APP_DISPLAY_NAME, APP_DOCUMENTATION_SITE
-from zenmapCore.BasePaths import fs_enc
 from zenmapCore.Paths import Path
 from zenmapCore.RecentScans import recent_scans
 from zenmapCore.UmitLogging import log
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
 import zenmapGUI.Print
 from zenmapCore.UmitConf import SearchConfig, is_maemo, WindowConfig, config_parser
-from zenmapCore.NetworkInventory import FilteredNetworkInventory
 
 UmitScanWindow = None
 hildon = None
@@ -170,22 +106,21 @@ if is_maemo():
             hildon.Window.__init__(self)
             self.set_resizable(False)
             self.set_border_width(0)
-            self.vbox = gtk.VBox()
+            self.vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
             self.vbox.set_border_width(0)
-            self.vbox.set_spacing(0)
 
 else:
     class UmitScanWindow(HIGMainWindow):
         def __init__(self):
             HIGMainWindow.__init__(self)
-            self.vbox = gtk.VBox()
+            self.vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
 
 
 def can_print():
     """Return true if we have printing operations (PyGTK 2.10 or later) or
     false otherwise."""
     try:
-        gtk.PrintOperation
+        Gtk.PrintOperation
     except AttributeError:
         return False
     else:
@@ -193,18 +128,32 @@ def can_print():
 
 
 class ScanWindow(UmitScanWindow):
+
+    dark_mode = None
+
     def __init__(self):
         UmitScanWindow.__init__(self)
 
         window = WindowConfig()
+        settings = Gtk.Settings.get_default()
+
+        if ScanWindow.dark_mode is None:
+            ScanWindow.dark_mode = (window.dark_mode == "True")
+            settings.set_property(
+                    "gtk-application-prefer-dark-theme",
+                    ScanWindow.dark_mode)
 
         self.set_title(_(APP_DISPLAY_NAME))
         self.move(window.x, window.y)
         self.set_default_size(window.width, window.height)
 
         self.scan_interface = ScanInterface()
+        # Update highlighting
+        output_viewer = self.scan_interface.scan_result.scan_result_notebook.nmap_output.nmap_output
+        output_viewer.nmap_highlight.set_dark_mode(
+                ScanWindow.dark_mode)
 
-        self.main_accel_group = gtk.AccelGroup()
+        self.main_accel_group = Gtk.AccelGroup()
 
         self.add_accel_group(self.main_accel_group)
 
@@ -222,12 +171,12 @@ class ScanWindow(UmitScanWindow):
     def _create_ui_manager(self):
         """Creates the UI Manager and a default set of actions, and builds
         the menus using those actions."""
-        self.ui_manager = gtk.UIManager()
+        self.ui_manager = Gtk.UIManager()
 
         # See info on ActionGroup at:
         # * http://www.pygtk.org/pygtk2reference/class-gtkactiongroup.html
         # * http://www.gtk.org/api/2.6/gtk/GtkActionGroup.html
-        self.main_action_group = gtk.ActionGroup('MainActionGroup')
+        self.main_action_group = Gtk.ActionGroup.new('MainActionGroup')
 
         # See info on Action at:
         # * http://www.pygtk.org/pygtk2reference/class-gtkaction.html
@@ -241,39 +190,33 @@ class ScanWindow(UmitScanWindow):
         #   _('Open the results of a previous scan'),
         #   lambda x: True)
 
-        # gtk.STOCK_ABOUT is only available in PyGTK 2.6 and later.
-        try:
-            about_icon = gtk.STOCK_ABOUT
-        except:
-            about_icon = None
-
         self.main_actions = [
             # Top level
             ('Scan', None, _('Sc_an'), None),
 
             ('Save Scan',
-                gtk.STOCK_SAVE,
+                Gtk.STOCK_SAVE,
                 _('_Save Scan'),
                 None,
                 _('Save current scan results'),
                 self._save_scan_results_cb),
 
             ('Save All Scans to Directory',
-                gtk.STOCK_SAVE,
+                Gtk.STOCK_SAVE,
                 _('Save All Scans to _Directory'),
                 "<Control><Alt>s",
                 _('Save all scans into a directory'),
                 self._save_to_directory_cb),
 
             ('Open Scan',
-                gtk.STOCK_OPEN,
+                Gtk.STOCK_OPEN,
                 _('_Open Scan'),
                 None,
                 _('Open the results of a previous scan'),
                 self._load_scan_results_cb),
 
             ('Append Scan',
-                gtk.STOCK_ADD,
+                Gtk.STOCK_ADD,
                 _('_Open Scan in This Window'),
                 None,
                 _('Append a saved scan to the list of scans in this window.'),
@@ -283,56 +226,56 @@ class ScanWindow(UmitScanWindow):
             ('Tools', None, _('_Tools'), None),
 
             ('New Window',
-                gtk.STOCK_NEW,
+                Gtk.STOCK_NEW,
                 _('_New Window'),
                 "<Control>N",
                 _('Open a new scan window'),
                 self._new_scan_cb),
 
             ('Close Window',
-                gtk.STOCK_CLOSE,
+                Gtk.STOCK_CLOSE,
                 _('Close Window'),
                 "<Control>w",
                 _('Close this scan window'),
                 self._exit_cb),
 
             ('Print...',
-                gtk.STOCK_PRINT,
+                Gtk.STOCK_PRINT,
                 _('Print...'),
                 None,
                 _('Print the current scan'),
                 self._print_cb),
 
             ('Quit',
-                gtk.STOCK_QUIT,
+                Gtk.STOCK_QUIT,
                 _('Quit'),
                 "<Control>q",
                 _('Quit the application'),
                 self._quit_cb),
 
             ('New Profile',
-                gtk.STOCK_JUSTIFY_LEFT,
+                Gtk.STOCK_JUSTIFY_LEFT,
                 _('New _Profile or Command'),
                 '<Control>p',
                 _('Create a new scan profile using the current command'),
                 self._new_scan_profile_cb),
 
             ('Search Scan',
-                gtk.STOCK_FIND,
+                Gtk.STOCK_FIND,
                 _('Search Scan Results'),
                 '<Control>f',
                 _('Search for a scan result'),
                 self._search_scan_result),
 
             ('Filter Hosts',
-                gtk.STOCK_FIND,
+                Gtk.STOCK_FIND,
                 _('Filter Hosts'),
                 '<Control>l',
                 _('Search for host by criteria'),
                 self._filter_cb),
 
             ('Edit Profile',
-                gtk.STOCK_PROPERTIES,
+                Gtk.STOCK_PROPERTIES,
                 _('_Edit Selected Profile'),
                 '<Control>e',
                 _('Edit selected scan profile'),
@@ -342,7 +285,7 @@ class ScanWindow(UmitScanWindow):
             ('Profile', None, _('_Profile'), None),
 
             ('Compare Results',
-                gtk.STOCK_DND_MULTIPLE,
+                Gtk.STOCK_DND_MULTIPLE,
                 _('Compare Results'),
                 "<Control>D",
                 _('Compare Scan Results using Diffies'),
@@ -353,7 +296,7 @@ class ScanWindow(UmitScanWindow):
             ('Help', None, _('_Help'), None),
 
             ('Report a bug',
-                gtk.STOCK_DIALOG_INFO,
+                Gtk.STOCK_DIALOG_INFO,
                 _('_Report a bug'),
                 '<Control>b',
                 _("Report a bug"),
@@ -361,7 +304,7 @@ class ScanWindow(UmitScanWindow):
                 ),
 
             ('About',
-                about_icon,
+                Gtk.STOCK_ABOUT,
                 _('_About'),
                 None,
                 _("About %s") % APP_DISPLAY_NAME,
@@ -369,11 +312,18 @@ class ScanWindow(UmitScanWindow):
                 ),
 
             ('Show Help',
-                gtk.STOCK_HELP,
+                Gtk.STOCK_HELP,
                 _('_Help'),
                 None,
                 _('Shows the application help'),
                 self._help_cb),
+
+            ('Toggle Dark Mode',
+                None,
+                _('Toggle Dark Mode'),
+                None,
+                None,
+                self._toggle_dark),
             ]
 
         # See info on UIManager at:
@@ -416,6 +366,7 @@ class ScanWindow(UmitScanWindow):
         <menu action='Profile'>
             <menuitem action='New Profile'/>
             <menuitem action='Edit Profile'/>
+            <menuitem action='Toggle Dark Mode'/>
         </menu>
 
         <menu action='Help'>
@@ -474,7 +425,7 @@ class ScanWindow(UmitScanWindow):
         log.debug(">>> Saving result into database...")
         try:
             scan_interface.inventory.save_to_db()
-        except Exception, e:
+        except Exception as e:
             alert = HIGAlertDialog(
                     message_format=_("Can't save to database"),
                     secondary_text=_("Can't store unsaved scans to the "
@@ -499,8 +450,8 @@ class ScanWindow(UmitScanWindow):
                         xml.sax.saxutils.quoteattr(scan))
 
                 self.main_actions.append(new_rscan)
-        else:
-            new_rscan_xml += "<separator />\n"
+
+        new_rscan_xml += "<separator />\n"
 
         self.default_ui %= new_rscan_xml
 
@@ -509,7 +460,7 @@ class ScanWindow(UmitScanWindow):
         menubar = self.ui_manager.get_widget('/menubar')
 
         if is_maemo():
-            menu = gtk.Menu()
+            menu = Gtk.Menu()
             for child in menubar.get_children():
                 child.reparent(menu)
             self.set_menu(menu)
@@ -538,7 +489,7 @@ class ScanWindow(UmitScanWindow):
 
         filename = None
         response = self._results_filechooser_dialog.run()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             filename = self._results_filechooser_dialog.get_filename()
         elif response == RESPONSE_OPEN_DIRECTORY:
             filename = self._results_filechooser_dialog.get_filename()
@@ -601,7 +552,7 @@ class ScanWindow(UmitScanWindow):
             try:
                 # Parse result
                 scan_interface.load_from_file(filename)
-            except Exception, e:
+            except Exception as e:
                 alert = HIGAlertDialog(message_format=_('Error loading file'),
                                        secondary_text=str(e))
                 alert.run()
@@ -660,20 +611,20 @@ class ScanWindow(UmitScanWindow):
             dlg = HIGDialog(
                     title="Choose a scan to save",
                     parent=self,
-                    flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                        gtk.STOCK_SAVE, gtk.RESPONSE_OK))
-            dlg.vbox.pack_start(gtk.Label(
+                    flags=Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                    buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                        Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+            dlg.vbox.pack_start(Gtk.Label.new(
                 "You have %u scans loaded in the current view.\n"
                 "Select the scan which you would like to save." % num_scans),
-                False)
-            scan_combo = gtk.combo_box_new_text()
+                False, True, 0)
+            scan_combo = Gtk.ComboBoxText()
             for scan in self.scan_interface.inventory.get_scans():
                 scan_combo.append_text(scan.nmap_command)
             scan_combo.set_active(0)
-            dlg.vbox.pack_start(scan_combo, False)
+            dlg.vbox.pack_start(scan_combo, False, True, 0)
             dlg.vbox.show_all()
-            if dlg.run() == gtk.RESPONSE_OK:
+            if dlg.run() == Gtk.ResponseType.OK:
                 selected = scan_combo.get_active()
                 dlg.destroy()
             else:
@@ -691,7 +642,7 @@ class ScanWindow(UmitScanWindow):
         response = self._save_results_filechooser_dialog.run()
 
         filename = None
-        if (response == gtk.RESPONSE_OK):
+        if (response == Gtk.ResponseType.OK):
             filename = self._save_results_filechooser_dialog.get_filename()
             format = self._save_results_filechooser_dialog.get_format()
             # add .xml to filename if there is no other extension
@@ -728,12 +679,12 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         # display a directory chooser dialog
         dir_chooser = SaveToDirectoryChooserDialog(
                 title=_("Choose a directory to save scans into"))
-        if dir_chooser.run() == gtk.RESPONSE_OK:
+        if dir_chooser.run() == Gtk.ResponseType.OK:
             self._save_all(self.scan_interface, dir_chooser.get_filename())
         dir_chooser.destroy()
 
     def _about_cb_response(self, dialog, response_id):
-        if response_id == gtk.RESPONSE_DELETE_EVENT:
+        if response_id == Gtk.ResponseType.DELETE_EVENT:
             self._about_dialog = None
         else:
             self._about_dialog.hide()
@@ -751,7 +702,7 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             filenames = scan_interface.inventory.save_to_dir(directory)
             for scan in scan_interface.inventory.get_scans():
                 scan.unsaved = False
-        except Exception, ex:
+        except Exception as ex:
             alert = HIGAlertDialog(message_format=_('Can\'t save file'),
                         secondary_text=str(ex))
             alert.run()
@@ -760,9 +711,18 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             scan_interface.saved_filename = directory
 
             # Saving recent scan information
-            for filename in filenames:
-                recent_scans.add_recent_scan(filename)
-            recent_scans.save()
+            try:
+                for filename in filenames:
+                    recent_scans.add_recent_scan(filename)
+                recent_scans.save()
+            except (OSError, IOError) as e:
+                alert = HIGAlertDialog(
+                        message_format=_(
+                            "Can't save recent scan information"),
+                        secondary_text=_(
+                            "Can't open file to write.\n%s") % str(e))
+                alert.run()
+                alert.destroy()
 
     def _save(self, scan_interface, saved_filename, selected_index,
             format="xml"):
@@ -773,7 +733,7 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             scan_interface.inventory.save_to_file(
                     saved_filename, selected_index, format)
             scan_interface.inventory.get_scans()[selected_index].unsaved = False  # noqa
-        except (OSError, IOError), e:
+        except (OSError, IOError) as e:
             alert = HIGAlertDialog(
                     message_format=_("Can't save file"),
                     secondary_text=_("Can't open file to write.\n%s") % str(e))
@@ -787,8 +747,17 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
 
             if format == "xml":
                 # Saving recent scan information
-                recent_scans.add_recent_scan(saved_filename)
-                recent_scans.save()
+                try:
+                    recent_scans.add_recent_scan(saved_filename)
+                    recent_scans.save()
+                except (OSError, IOError) as e:
+                    alert = HIGAlertDialog(
+                            message_format=_(
+                                "Can't save recent scan information"),
+                            secondary_text=_(
+                                "Can't open file to write.\n%s") % str(e))
+                    alert.run()
+                    alert.destroy()
 
     def get_empty_interface(self):
         """Return this window if it is empty, otherwise create and return a new
@@ -818,7 +787,19 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         pe.show_all()
 
     def _help_cb(self, action):
-        show_help()
+        self.show_help()
+
+    def _toggle_dark(self, action):
+        ScanWindow.dark_mode = not ScanWindow.dark_mode
+        settings = Gtk.Settings.get_default()
+        settings.set_property("gtk-application-prefer-dark-theme",
+            ScanWindow.dark_mode)
+        # Update highlighting
+        output_viewer = self.scan_interface.scan_result.scan_result_notebook.nmap_output.nmap_output
+        output_viewer.nmap_highlight.set_dark_mode(
+                ScanWindow.dark_mode)
+        output_viewer.apply_highlighting()
+
 
     def _exit_cb(self, *args):
         """Closes the window, prompting for confirmation if necessary. If one
@@ -827,9 +808,9 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         if self.scan_interface.changed:
             log.debug("Found changes on closing window")
             dialog = HIGDialog(
-                    buttons=(_('Close anyway').encode('utf-8'),
-                        gtk.RESPONSE_CLOSE, gtk.STOCK_CANCEL,
-                        gtk.RESPONSE_CANCEL))
+                    buttons=(_('Close anyway'),
+                        Gtk.ResponseType.CLOSE, Gtk.STOCK_CANCEL,
+                        Gtk.ResponseType.CANCEL))
 
             alert = HIGEntryLabel('<b>%s</b>' % _("Unsaved changes"))
 
@@ -843,22 +824,22 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             vbox.set_border_width(5)
             vbox.set_spacing(12)
 
-            image = gtk.Image()
+            image = Gtk.Image()
             image.set_from_stock(
-                    gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
+                    Gtk.STOCK_DIALOG_QUESTION, Gtk.IconSize.DIALOG)
 
-            vbox.pack_start(alert)
-            vbox.pack_start(text)
-            hbox.pack_start(image)
-            hbox.pack_start(vbox)
+            vbox.pack_start(alert, True, True, 0)
+            vbox.pack_start(text, True, True, 0)
+            hbox.pack_start(image, True, True, 0)
+            hbox.pack_start(vbox, True, True, 0)
 
-            dialog.vbox.pack_start(hbox)
+            dialog.vbox.pack_start(hbox, True, True, 0)
             dialog.vbox.show_all()
 
             response = dialog.run()
             dialog.destroy()
 
-            if response == gtk.RESPONSE_CANCEL:
+            if response == Gtk.ResponseType.CANCEL:
                 return True
 
             search_config = SearchConfig()
@@ -868,9 +849,9 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         elif self.scan_interface.num_scans_running() > 0:
             log.debug("Trying to close a window with a running scan")
             dialog = HIGDialog(
-                    buttons=(_('Close anyway').encode('utf-8'),
-                        gtk.RESPONSE_CLOSE, gtk.STOCK_CANCEL,
-                        gtk.RESPONSE_CANCEL))
+                    buttons=(_('Close anyway'),
+                        Gtk.ResponseType.CLOSE, Gtk.STOCK_CANCEL,
+                        Gtk.ResponseType.CANCEL))
 
             alert = HIGEntryLabel('<b>%s</b>' % _("Trying to close"))
 
@@ -885,29 +866,30 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             vbox.set_border_width(5)
             vbox.set_spacing(12)
 
-            image = gtk.Image()
+            image = Gtk.Image()
             image.set_from_stock(
-                    gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
+                    Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.DIALOG)
 
-            vbox.pack_start(alert)
-            vbox.pack_start(text)
-            hbox.pack_start(image)
-            hbox.pack_start(vbox)
+            vbox.pack_start(alert, True, True, 0)
+            vbox.pack_start(text, True, True, 0)
+            hbox.pack_start(image, True, True, 0)
+            hbox.pack_start(vbox, True, True, 0)
 
-            dialog.vbox.pack_start(hbox)
+            dialog.vbox.pack_start(hbox, True, True, 0)
             dialog.vbox.show_all()
 
             response = dialog.run()
             dialog.destroy()
 
-            if response == gtk.RESPONSE_CLOSE:
+            if response == Gtk.ResponseType.CLOSE:
                 self.scan_interface.kill_all_scans()
-            elif response == gtk.RESPONSE_CANCEL:
+            elif response == Gtk.ResponseType.CANCEL:
                 return True
 
         window = WindowConfig()
         window.x, window.y = self.get_position()
         window.width, window.height = self.get_size()
+        window.dark_mode = ScanWindow.dark_mode
         window.save_changes()
         if config_parser.failed:
             alert = HIGAlertDialog(
@@ -948,29 +930,26 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         self.diff_window.show_all()
 
 
-def show_help():
-    import urllib
-    import webbrowser
+    def show_help(self):
+        import urllib.request
+        import webbrowser
 
-    new = 0
-    if sys.hexversion >= 0x2050000:
-        new = 2
+        doc_path = abspath(join(Path.docs_dir, "help.html"))
+        url = "file:" + urllib.request.pathname2url(doc_path)
 
-    doc_path = abspath(join(Path.docs_dir, "help.html"))
-    url = "file:" + urllib.pathname2url(fs_enc(doc_path))
-    try:
-        webbrowser.open(url, new=new)
-    except OSError, e:
-        d = HIGAlertDialog(parent=self,
-                           message_format=_("Can't find documentation files"),
-                           secondary_text=_("""\
+        try:
+            webbrowser.open(url, new=2)
+        except OSError as e:
+            d = HIGAlertDialog(parent=self,
+                               message_format=_("Can't find documentation files"),
+                               secondary_text=_("""\
 There was an error loading the documentation file %s (%s). See the \
 online documentation at %s.\
-""") % (doc_path, unicode(e), APP_DOCUMENTATION_SITE))
-        d.run()
-        d.destroy()
+""") % (doc_path, str(e), APP_DOCUMENTATION_SITE))
+            d.run()
+            d.destroy()
 
 if __name__ == '__main__':
     w = ScanWindow()
     w.show_all()
-    gtk.main()
+    Gtk.main()
